import re

import pytest
from playwright.sync_api import expect

pytestmark = pytest.mark.django_db


def test_scale_control(map, live_server, datalayer, page):
    control = page.locator(".leaflet-control-scale")
    page.goto(f"{live_server.url}{map.get_absolute_url()}")
    expect(control).to_be_visible()
    page.goto(f"{live_server.url}{map.get_absolute_url()}?scaleControl=false")
    expect(control).to_be_hidden()


def test_datalayers_control(map, live_server, datalayer, page):
    control = page.locator(".umap-control-browse")
    browser = page.locator(".umap-browser")
    page.goto(f"{live_server.url}{map.get_absolute_url()}")
    expect(control).to_be_visible()
    expect(browser).to_be_hidden()
    page.goto(f"{live_server.url}{map.get_absolute_url()}?datalayersControl=true")
    expect(control).to_be_visible()
    expect(browser).to_be_hidden()
    page.goto(f"{live_server.url}{map.get_absolute_url()}?datalayersControl=null")
    expect(control).to_be_hidden()
    expect(browser).to_be_hidden()
    page.goto(f"{live_server.url}{map.get_absolute_url()}?datalayersControl=false")
    expect(control).to_be_hidden()
    expect(browser).to_be_hidden()
    # Retrocompat
    page.goto(f"{live_server.url}{map.get_absolute_url()}?datalayersControl=expanded")
    expect(control).to_be_visible()
    expect(browser).to_be_visible()
    # Should not override onLoadPanel
    page.goto(
        f"{live_server.url}{map.get_absolute_url()}?datalayersControl=expanded&onLoadPanel=caption"
    )
    expect(control).to_be_visible()
    expect(browser).to_be_hidden()
    expect(page.locator(".umap-caption")).to_be_visible()


def test_can_deactivate_wheel_from_query_string(map, live_server, page):
    page.goto(f"{live_server.url}{map.get_absolute_url()}")
    expect(page).to_have_url(re.compile(r".*#7/.+"))
    page.mouse.wheel(0, 1)
    expect(page).to_have_url(re.compile(r".*#6/.+"))
    page.goto(f"{live_server.url}{map.get_absolute_url()}?scrollWheelZoom=false")
    expect(page).to_have_url(re.compile(r".*#7/.+"))
    page.mouse.wheel(0, 1)
    expect(page).to_have_url(re.compile(r".*#7/.+"))


def test_zoom_control(map, live_server, datalayer, page):
    control = page.locator(".leaflet-control-zoom")
    page.goto(f"{live_server.url}{map.get_absolute_url()}")
    expect(control).to_be_visible()
    page.goto(f"{live_server.url}{map.get_absolute_url()}?zoomControl=false")
    expect(control).to_be_hidden()
    page.goto(f"{live_server.url}{map.get_absolute_url()}?zoomControl=true")
    expect(control).to_be_visible()
    page.goto(f"{live_server.url}{map.get_absolute_url()}?zoomControl=null")
    expect(control).to_be_hidden()
    page.get_by_title("More controls").click()
    expect(control).to_be_visible()



import re

import pytest
from playwright.sync_api import expect

from umap.models import TileLayer

from ..base import TileLayerFactory

pytestmark = pytest.mark.django_db


@pytest.fixture
def tilelayers():
    # Create one more TileLayer than what we display in the switcher (11 vs 10)
    TileLayerFactory(
        rank=1,
        name="OpenStreetMap",
        url_template="https://tile.openstreetmap.org/{z}/{x}/{y}.png",
    )
    TileLayerFactory(
        rank=2,
        name="Forte",
        url_template="https://{s}.forte.tiles.quaidorsay.fr/fr{r}/{z}/{x}/{y}.png",
    )
    TileLayerFactory(
        rank=3,
        name="Positron",
        url_template="https://cartodb-basemaps-{s}.global.ssl.fastly.net/light_all/{z}/{x}/{y}.png",
    )
    TileLayerFactory(
        rank=4,
        name="Humanitarian",
        url_template="//{s}.tile.openstreetmap.fr/hot/{z}/{x}/{y}.png",
    )
    TileLayerFactory(
        rank=5,
        name="Dark Matter",
        url_template="https://cartodb-basemaps-{s}.global.ssl.fastly.net/dark_all/{z}/{x}/{y}.png",
    )
    TileLayerFactory(
        rank=6,
        name="OSM OpenCycleMap",
        url_template="https://{s}.tile.thunderforest.com/cycle/{z}/{x}/{y}.png?apikey=e6b144cfc47a48fd928dad578eb026a6",
    )
    TileLayerFactory(
        rank=7,
        name="CyclOSM",
        url_template="//{s}.tile-cyclosm.openstreetmap.fr/cyclosm/{z}/{x}/{y}.png",
    )
    TileLayerFactory(
        rank=8,
        name="Piano",
        url_template="https://{s}.piano.tiles.quaidorsay.fr/fr{r}/{z}/{x}/{y}.png",
    )
    TileLayerFactory(
        rank=9,
        name="IGN Image aérienne (France)",
        url_template="https://wxs.ign.fr/pratique/wmts/?SERVICE=WMTS&REQUEST=GetTile&VERSION=1.0.0&LAYER=ORTHOIMAGERY.ORTHOPHOTOS&STYLE=normal&TILEMATRIXSET=PM&TILEMATRIX={z}&TILEROW={y}&TILECOL={x}&FORMAT=image%2Fjpeg",
    )
    TileLayerFactory(
        rank=10,
        name="OSM OpenRiverboatMap",
        url_template="//{s}.tile.openstreetmap.fr/openriverboatmap/{z}/{x}/{y}.png",
    )
    TileLayerFactory(
        rank=11,
        name="OSM OpenTopoMap",
        url_template="https://{s}.tile.opentopomap.org/{z}/{x}/{y}.png",
    )


def test_map_should_display_first_tilelayer_by_default(
    map, live_server, tilelayers, page
):
    page.goto(f"{live_server.url}/map/new")
    tiles = page.locator(".leaflet-tile-pane img")
    expect(tiles.first).to_have_attribute(
        "src", re.compile(r"https://tile.openstreetmap.org/\d+/\d+/\d+.png")
    )


def test_map_should_display_selected_tilelayer(map, live_server, tilelayers, page):
    piano = TileLayer.objects.get(name="Piano")
    url_pattern = re.compile(
        r"https://[abc]{1}.piano.tiles.quaidorsay.fr/fr/\d+/\d+/\d+.png"
    )
    map.settings["properties"]["tilelayer"]["url_template"] = piano.url_template
    map.settings["properties"]["tilelayersControl"] = True
    map.save()
    page.goto(f"{live_server.url}{map.get_absolute_url()}")
    tiles = page.locator(".leaflet-tile-pane img")
    expect(tiles.first).to_have_attribute("src", url_pattern)
    iconTiles = page.locator(".leaflet-iconLayers .leaflet-iconLayers-layer")
    # The second of the list should be the current
    expect(iconTiles.nth(1)).to_have_css("background-image", url_pattern)


def test_map_should_display_custom_tilelayer(map, live_server, tilelayers, page):
    # Add one not on the list
    url_pattern = re.compile(
        r"https://[abc]{1}.basemaps.cartocdn.com/rastertiles/voyager/\d+/\d+/\d+.png"
    )
    map.settings["properties"]["tilelayer"]["url_template"] = (
        "https://{s}.basemaps.cartocdn.com/rastertiles/voyager/{z}/{x}/{y}{r}.png"
    )
    map.settings["properties"]["tilelayersControl"] = True
    map.save()
    page.goto(f"{live_server.url}{map.get_absolute_url()}")
    tiles = page.locator(".leaflet-tile-pane img")
    expect(tiles.first).to_have_attribute("src", url_pattern)
    iconTiles = page.locator(".leaflet-iconLayers .leaflet-iconLayers-layer")
    # The second of the list should be the current
    expect(iconTiles.nth(1)).to_have_css("background-image", url_pattern)


def test_can_have_smart_text_in_attribution(tilelayer, map, live_server, page):
    map.settings["properties"]["tilelayer"]["attribution"] = (
        "&copy; [[http://www.openstreetmap.org/copyright|OpenStreetMap]] contributors"
    )
    map.save()
    page.goto(f"{live_server.url}{map.get_absolute_url()}")
    expect(page.get_by_text("© OpenStreetMap contributors")).to_be_visible()
    expect(page.get_by_role("link", name="OpenStreetMap")).to_be_visible()


def test_map_should_display_a_more_button(map, live_server, tilelayers, page):
    map.settings["properties"]["tilelayersControl"] = True
    map.save()
    page.goto(f"{live_server.url}{map.get_absolute_url()}")
    page.locator(".leaflet-iconLayers").hover()
    page.get_by_role("button", name="+").click()
    panel = page.locator(".panel.left.on")
    expect(panel).to_be_visible()
    expect(panel.get_by_text("Forte")).to_be_visible()
    panel.get_by_text("Forte").click()
    tiles = page.locator(".leaflet-tile-pane img")
    url_pattern = re.compile(
        r"https://[abc]{1}.forte.tiles.quaidorsay.fr/fr/\d+/\d+/\d+.png"
    )
    expect(tiles.first).to_have_attribute("src", url_pattern)



# Generated by Django 4.1.7 on 2023-05-05 18:02

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("umap", "0008_alter_map_settings"),
    ]

    operations = [
        migrations.CreateModel(
            name="Star",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("at", models.DateTimeField(auto_now=True)),
                (
                    "by",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="stars",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                (
                    "map",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE, to="umap.map"
                    ),
                ),
            ],
        ),
    ]



import pytest
from playwright.sync_api import expect

from ..base import DataLayerFactory

pytestmark = pytest.mark.django_db


DATALAYER_DATA = {
    "type": "FeatureCollection",
    "features": [
        {
            "type": "Feature",
            "geometry": {
                "type": "Point",
                "coordinates": [13.6, 48.5],
            },
            "properties": {"name": "1st Point"},
        },
        {
            "type": "Feature",
            "geometry": {
                "type": "Point",
                "coordinates": [13.7, 48.4],
            },
            "properties": {"name": "2d Point"},
        },
        {
            "type": "Feature",
            "geometry": {
                "type": "Point",
                "coordinates": [13.5, 48.6],
            },
            "properties": {"name": "3d Point"},
        },
    ],
}


def test_can_use_slideshow_manually(map, live_server, page):
    map.settings["properties"]["slideshow"] = {"active": True, "delay": 5000}
    map.save()
    DataLayerFactory(map=map, data=DATALAYER_DATA)
    page.goto(f"{live_server.url}{map.get_absolute_url()}")
    first_point = page.get_by_text("1st Point")
    second_point = page.get_by_text("2d Point")
    third_point = page.get_by_text("3d Point")
    expect(first_point).to_be_hidden()
    expect(second_point).to_be_hidden()
    expect(third_point).to_be_hidden()
    next_ = page.get_by_title("Zoom to the next")
    expect(next_).to_be_visible()
    next_.click()
    expect(first_point).to_be_visible()
    next_.click()
    expect(first_point).to_be_hidden()
    expect(second_point).to_be_visible()
    next_.click()
    expect(first_point).to_be_hidden()
    expect(second_point).to_be_hidden()
    expect(third_point).to_be_visible()
    next_.click()
    expect(first_point).to_be_visible()
    expect(second_point).to_be_hidden()
    expect(third_point).to_be_hidden()



import copy
import re

import pytest
from playwright.sync_api import expect

from ..base import DataLayerFactory

pytestmark = pytest.mark.django_db


DATALAYER_DATA1 = {
    "type": "FeatureCollection",
    "features": [
        {
            "type": "Feature",
            "properties": {
                "mytype": "even",
                "name": "Point 2",
                "mynumber": 10,
                "mydate": "2024/04/14 12:19:17",
            },
            "geometry": {"type": "Point", "coordinates": [0.065918, 48.385442]},
        },
        {
            "type": "Feature",
            "properties": {
                "mytype": "odd",
                "name": "Point 1",
                "mynumber": 12,
                "mydate": "2024/03/13 12:20:20",
            },
            "geometry": {"type": "Point", "coordinates": [3.55957, 49.767074]},
        },
    ],
    "_umap_options": {
        "name": "Calque 1",
    },
}


DATALAYER_DATA2 = {
    "type": "FeatureCollection",
    "features": [
        {
            "type": "Feature",
            "properties": {
                "mytype": "even",
                "name": "Point 4",
                "mynumber": 10,
                "mydate": "2024/08/18 13:14:15",
            },
            "geometry": {"type": "Point", "coordinates": [0.856934, 45.290347]},
        },
        {
            "type": "Feature",
            "properties": {
                "mytype": "odd",
                "name": "Point 3",
                "mynumber": 14,
                "mydate": "2024-04-14T10:19:17.000Z",
            },
            "geometry": {"type": "Point", "coordinates": [4.372559, 47.945786]},
        },
    ],
    "_umap_options": {
        "name": "Calque 2",
    },
}


DATALAYER_DATA3 = {
    "type": "FeatureCollection",
    "features": [
        {
            "type": "Feature",
            "properties": {"name": "a polygon"},
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [2.12, 49.57],
                        [1.08, 49.02],
                        [2.51, 47.55],
                        [3.19, 48.77],
                        [2.12, 49.57],
                    ]
                ],
            },
        },
    ],
    "_umap_options": {"name": "Calque 2", "browsable": False},
}


def test_simple_facet_search(live_server, page, map):
    map.settings["properties"]["onLoadPanel"] = "datafilters"
    map.settings["properties"]["facetKey"] = "mytype|My type,mynumber|My Number|number"
    map.settings["properties"]["showLabel"] = True
    map.save()
    DataLayerFactory(map=map, data=DATALAYER_DATA1)
    DataLayerFactory(map=map, data=DATALAYER_DATA2)
    DataLayerFactory(map=map, data=DATALAYER_DATA3)
    page.goto(f"{live_server.url}{map.get_absolute_url()}#6/48.948/1.670")
    panel = page.locator(".panel.left.on")
    expect(panel).to_have_class(re.compile(".*expanded.*"))
    expect(panel.locator(".umap-browser")).to_be_visible()
    # From a non browsable datalayer, should not be impacted
    paths = page.locator(".leaflet-overlay-pane path")
    expect(paths).to_be_visible()
    expect(panel).to_be_visible()
    # Facet name
    expect(page.get_by_text("My type")).to_be_visible()
    # Facet values
    oven = page.get_by_text("even")
    odd = page.get_by_text("odd")
    expect(oven).to_be_visible()
    expect(odd).to_be_visible()
    expect(paths).to_be_visible()
    markers = page.locator(".leaflet-marker-icon")
    expect(markers).to_have_count(4)
    # Tooltips
    expect(page.get_by_role("tooltip", name="Point 1")).to_be_visible()
    expect(page.get_by_role("tooltip", name="Point 2")).to_be_visible()
    expect(page.get_by_role("tooltip", name="Point 3")).to_be_visible()
    expect(page.get_by_role("tooltip", name="Point 4")).to_be_visible()

    # Datalist
    expect(panel.get_by_text("Point 1")).to_be_visible()
    expect(panel.get_by_text("Point 2")).to_be_visible()
    expect(panel.get_by_text("Point 3")).to_be_visible()
    expect(panel.get_by_text("Point 4")).to_be_visible()

    # Now let's filter
    odd.click()
    expect(markers).to_have_count(2)
    expect(page.get_by_role("tooltip", name="Point 2")).to_be_hidden()
    expect(page.get_by_role("tooltip", name="Point 4")).to_be_hidden()
    expect(page.get_by_role("tooltip", name="Point 1")).to_be_visible()
    expect(page.get_by_role("tooltip", name="Point 3")).to_be_visible()
    expect(panel.get_by_text("Point 2")).to_be_hidden()
    expect(panel.get_by_text("Point 4")).to_be_hidden()
    expect(panel.get_by_text("Point 1")).to_be_visible()
    expect(panel.get_by_text("Point 3")).to_be_visible()
    expect(paths).to_be_visible
    # Now let's filter
    odd.click()
    expect(markers).to_have_count(4)
    expect(paths).to_be_visible()

    # Let's filter using the number facet
    expect(page.get_by_text("My Number")).to_be_visible()
    expect(page.get_by_label("Min")).to_have_value("10")
    expect(page.get_by_label("Max")).to_have_value("14")
    page.get_by_label("Min").fill("11")
    page.keyboard.press("Tab")  # Move out of the input, so the "change" event is sent
    expect(markers).to_have_count(2)
    expect(paths).to_be_visible()
    page.get_by_label("Max").fill("13")
    page.keyboard.press("Tab")
    expect(markers).to_have_count(1)

    # Now let's combine
    page.get_by_label("Min").fill("10")
    page.keyboard.press("Tab")
    expect(markers).to_have_count(3)
    odd.click()
    expect(markers).to_have_count(1)
    expect(paths).to_be_visible()


def test_date_facet_search(live_server, page, map):
    map.settings["properties"]["onLoadPanel"] = "datafilters"
    map.settings["properties"]["facetKey"] = "mydate|Date filter|date"
    map.save()
    DataLayerFactory(map=map, data=DATALAYER_DATA1)
    DataLayerFactory(map=map, data=DATALAYER_DATA2)
    page.goto(f"{live_server.url}{map.get_absolute_url()}#6/47.5/-1.5")
    markers = page.locator(".leaflet-marker-icon")
    expect(markers).to_have_count(4)
    expect(page.get_by_text("Date Filter")).to_be_visible()
    expect(page.get_by_label("From")).to_have_value("2024-03-13")
    expect(page.get_by_label("Until")).to_have_value("2024-08-18")
    page.get_by_label("From").fill("2024-03-14")
    expect(markers).to_have_count(3)
    page.get_by_label("Until").fill("2024-08-17")
    expect(markers).to_have_count(2)


def test_choice_with_empty_value(live_server, page, map):
    map.settings["properties"]["onLoadPanel"] = "datafilters"
    map.settings["properties"]["facetKey"] = "mytype|My type"
    map.save()
    data = copy.deepcopy(DATALAYER_DATA1)
    data["features"][0]["properties"]["mytype"] = ""
    del data["features"][1]["properties"]["mytype"]
    DataLayerFactory(map=map, data=data)
    DataLayerFactory(map=map, data=DATALAYER_DATA2)
    page.goto(f"{live_server.url}{map.get_absolute_url()}#6/47.5/-1.5")
    expect(page.get_by_text("<empty value>")).to_be_visible()
    markers = page.locator(".leaflet-marker-icon")
    expect(markers).to_have_count(4)
    page.get_by_text("<empty value>").click()
    expect(markers).to_have_count(2)


def test_number_with_zero_value(live_server, page, map):
    map.settings["properties"]["onLoadPanel"] = "datafilters"
    map.settings["properties"]["facetKey"] = "mynumber|Filter|number"
    map.save()
    data = copy.deepcopy(DATALAYER_DATA1)
    data["features"][0]["properties"]["mynumber"] = 0
    DataLayerFactory(map=map, data=data)
    DataLayerFactory(map=map, data=DATALAYER_DATA2)
    page.goto(f"{live_server.url}{map.get_absolute_url()}#6/47.5/-1.5")
    expect(page.get_by_label("Min")).to_have_value("0")
    expect(page.get_by_label("Max")).to_have_value("14")
    page.get_by_label("Min").fill("1")
    page.keyboard.press("Tab")  # Move out of the input, so the "change" event is sent
    markers = page.locator(".leaflet-marker-icon")
    expect(markers).to_have_count(3)


def test_facets_search_are_persistent_when_closing_panel(live_server, page, map):
    map.settings["properties"]["onLoadPanel"] = "datafilters"
    map.settings["properties"]["facetKey"] = "mytype|My type,mynumber|My Number|number"
    map.save()
    DataLayerFactory(map=map, data=DATALAYER_DATA1)
    DataLayerFactory(map=map, data=DATALAYER_DATA2)
    page.goto(f"{live_server.url}{map.get_absolute_url()}#6/48.948/1.670")
    panel = page.locator(".panel.left")

    # Facet values
    odd = page.get_by_label("odd")
    markers = page.locator(".leaflet-marker-icon")
    expect(markers).to_have_count(4)

    # Datalist in the browser
    expect(panel.get_by_text("Point 1")).to_be_visible()
    expect(panel.get_by_text("Point 2")).to_be_visible()
    expect(panel.get_by_text("Point 3")).to_be_visible()
    expect(panel.get_by_text("Point 4")).to_be_visible()

    # Now let's filter
    odd.click()
    expect(page.locator("summary")).to_have_attribute("data-badge", " ")
    expect(page.locator(".umap-control-browse")).to_have_attribute("data-badge", " ")
    expect(markers).to_have_count(2)
    expect(panel.get_by_text("Point 2")).to_be_hidden()
    expect(panel.get_by_text("Point 4")).to_be_hidden()
    expect(panel.get_by_text("Point 1")).to_be_visible()
    expect(panel.get_by_text("Point 3")).to_be_visible()

    # Let's filter using the number facet
    expect(panel.get_by_label("Min")).to_have_value("10")
    expect(panel.get_by_label("Max")).to_have_value("14")
    page.get_by_label("Min").fill("13")
    page.keyboard.press("Tab")  # Move out of the input, so the "change" event is sent
    expect(panel.get_by_label("Min")).to_have_attribute("data-modified", "true")
    expect(markers).to_have_count(1)
    expect(panel.get_by_text("Point 2")).to_be_hidden()
    expect(panel.get_by_text("Point 4")).to_be_hidden()
    expect(panel.get_by_text("Point 1")).to_be_hidden()
    expect(panel.get_by_text("Point 3")).to_be_visible()

    # Close panel
    expect(panel.locator("summary")).to_have_attribute("data-badge", " ")
    expect(page.locator(".umap-control-browse")).to_have_attribute("data-badge", " ")
    panel.get_by_role("button", name="Close").click()
    page.get_by_role("button", name="Open browser").click()
    expect(panel.get_by_label("Min")).to_have_value("13")
    expect(panel.get_by_label("Min")).to_have_attribute("data-modified", "true")
    expect(panel.get_by_label("odd")).to_be_checked()

    # Datalist in the browser should be inchanged
    expect(panel.get_by_text("Point 2")).to_be_hidden()
    expect(panel.get_by_text("Point 4")).to_be_hidden()
    expect(panel.get_by_text("Point 1")).to_be_hidden()
    expect(panel.get_by_text("Point 3")).to_be_visible()



from gzip import GzipFile

from django.core.exceptions import ImproperlyConfigured

try:
    from botocore.exceptions import ClientError
    from storages.backends.s3 import S3Storage
except ImportError:
    raise ImproperlyConfigured(
        "You need to install s3 dependencies: pip install umap-project[s3]"
    )


class S3DataStorage(S3Storage):
    gzip = True

    def get_reference_version(self, instance):
        metadata = self.connection.meta.client.head_object(
            Bucket=self.bucket_name, Key=instance.geojson.name
        )
        # Do not fail if bucket does not handle versioning
        return metadata.get("VersionId", metadata["ETag"])

    def make_filename(self, instance):
        return f"{str(instance.pk)}.geojson"

    def list_versions(self, instance):
        response = self.connection.meta.client.list_object_versions(
            Bucket=self.bucket_name, Prefix=instance.geojson.name
        )
        return [
            {
                "ref": version["VersionId"],
                "at": version["LastModified"].timestamp() * 1000,
                "size": version["Size"],
            }
            for version in response["Versions"]
        ]

    def get_version(self, ref, instance):
        try:
            data = self.connection.meta.client.get_object(
                Bucket=self.bucket_name,
                Key=instance.geojson.name,
                VersionId=ref,
            )
        except ClientError:
            raise ValueError(f"Invalid version reference: {ref}")
        return GzipFile(mode="r", fileobj=data["Body"]).read()

    def get_version_path(self, ref, instance):
        return self.url(instance.geojson.name, parameters={"VersionId": ref})

    def onDatalayerSave(self, instance):
        pass

    def onDatalayerDelete(self, instance):
        return self.connection.meta.client.delete_object(
            Bucket=self.bucket_name,
            Key=instance.geojson.name,
        )



import platform
from copy import deepcopy

import pytest
from playwright.sync_api import expect

from ..base import DataLayerFactory

pytestmark = pytest.mark.django_db

DATALAYER_DATA = {
    "type": "FeatureCollection",
    "features": [
        {
            "type": "Feature",
            "properties": {
                "name": "test marker",
                "description": "Some description",
            },
            "id": "QwNjg",
            "geometry": {
                "type": "Point",
                "coordinates": [14.6889, 48.5529],
            },
        },
    ],
}


@pytest.fixture
def bootstrap(map, live_server):
    DataLayerFactory(map=map, data=DATALAYER_DATA)


def test_can_edit_on_shift_click(live_server, openmap, page, datalayer):
    modifier = "Meta" if platform.system() == "Darwin" else "Control"
    page.goto(f"{live_server.url}{openmap.get_absolute_url()}?edit")
    page.locator(".leaflet-marker-icon").click(modifiers=[modifier, "Shift"])
    expect(page.get_by_text("Layer properties")).to_be_visible()


def test_marker_style_should_have_precedence(live_server, openmap, page, bootstrap):
    page.goto(f"{live_server.url}{openmap.get_absolute_url()}?edit#6/48.5/19")

    # Change colour at layer level
    page.get_by_role("button", name="Manage layers").click()
    page.locator(".panel").get_by_title("Edit", exact=True).click()
    page.get_by_text("Shape properties").click()
    page.locator(".umap-field-color .define").click()
    expect(page.locator(".leaflet-marker-icon .icon_container")).to_have_css(
        "background-color", "rgb(0, 0, 139)"
    )
    page.get_by_title("DarkRed").first.click()
    expect(page.locator(".leaflet-marker-icon .icon_container")).to_have_css(
        "background-color", "rgb(139, 0, 0)"
    )

    # Now change at marker level, it should take precedence
    page.locator(".leaflet-marker-icon").click(modifiers=["Shift"])
    page.get_by_text("Shape properties").click()
    page.locator("#umap-feature-shape-properties").get_by_text("define").first.click()
    page.get_by_title("GoldenRod", exact=True).click()
    expect(page.locator(".leaflet-marker-icon .icon_container")).to_have_css(
        "background-color", "rgb(218, 165, 32)"
    )

    # Now change again at layer level again, it should not change the marker color
    page.get_by_role("button", name="Manage layers").click()
    page.locator(".panel").get_by_title("Edit", exact=True).click()
    page.get_by_text("Shape properties").click()
    page.locator(".umap-field-color input").click()
    page.get_by_title("DarkViolet").first.click()
    expect(page.locator(".leaflet-marker-icon .icon_container")).to_have_css(
        "background-color", "rgb(218, 165, 32)"
    )


def test_should_open_an_edit_toolbar_on_click(live_server, openmap, page, bootstrap):
    page.goto(f"{live_server.url}{openmap.get_absolute_url()}?edit")
    page.locator(".leaflet-marker-icon").click()
    expect(page.get_by_role("button", name="Toggle edit mode")).to_be_visible()
    expect(page.get_by_role("button", name="Delete this feature")).to_be_visible()


def test_should_update_open_popup_on_edit(live_server, openmap, page, bootstrap):
    page.goto(f"{live_server.url}{openmap.get_absolute_url()}")
    expect(page.locator(".umap-icon-active")).to_be_hidden()
    page.locator(".leaflet-marker-icon").click()
    expect(page.locator(".leaflet-popup-content-wrapper")).to_be_visible()
    expect(page.get_by_text("test marker")).to_be_visible()
    expect(page.get_by_text("Some description")).to_be_visible()
    page.get_by_role("button", name="Edit").click()
    page.locator(".leaflet-marker-icon").click(modifiers=["Shift"])
    page.locator('input[name="name"]').fill("test marker edited")
    expect(page.get_by_text("test marker edited")).to_be_visible()


def test_should_follow_datalayer_style_when_changing_datalayer(
    live_server, openmap, page
):
    data = deepcopy(DATALAYER_DATA)
    data["_umap_options"] = {"color": "DarkCyan"}
    DataLayerFactory(map=openmap, data=data)
    DataLayerFactory(
        map=openmap,
        name="other datalayer",
        data={
            "type": "FeatureCollection",
            "features": [],
            "_umap_options": {"color": "DarkViolet"},
        },
    )
    page.goto(f"{live_server.url}{openmap.get_absolute_url()}?edit")
    marker = page.locator(".leaflet-marker-icon .icon_container")
    expect(marker).to_have_css("background-color", "rgb(0, 139, 139)")
    # Change datalayer
    marker.click()
    page.get_by_role("button", name="Toggle edit mode (⇧+Click)").click()
    page.locator(".umap-field-datalayer select").select_option(label="other datalayer")
    expect(marker).to_have_css("background-color", "rgb(148, 0, 211)")



import json
from pathlib import Path

import pytest
from playwright.sync_api import expect

from ..base import DataLayerFactory

pytestmark = pytest.mark.django_db

DATALAYER_DATA = {
    "type": "FeatureCollection",
    "features": [
        {
            "type": "Feature",
            "properties": {
                "name": "name poly",
            },
            "id": "gyNzM",
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [11.25, 53.585984],
                        [10.151367, 52.975108],
                        [12.689209, 52.167194],
                        [14.084473, 53.199452],
                        [12.634277, 53.618579],
                        [11.25, 53.585984],
                        [11.25, 53.585984],
                    ],
                ],
            },
        },
        {
            "type": "Feature",
            "properties": {
                "_umap_options": {
                    "color": "OliveDrab",
                },
                "name": "test",
                "description": "Some description",
            },
            "id": "QwNjg",
            "geometry": {
                "type": "Point",
                "coordinates": [-0.274658, 52.57635],
            },
        },
        {
            "type": "Feature",
            "properties": {
                "_umap_options": {
                    "fill": False,
                    "opacity": 0.6,
                },
                "name": "test",
            },
            "id": "YwMTM",
            "geometry": {
                "type": "LineString",
                "coordinates": [
                    [-0.571289, 54.476422],
                    [0.439453, 54.610255],
                    [1.724854, 53.448807],
                    [4.163818, 53.988395],
                    [5.306396, 53.533778],
                    [6.591797, 53.709714],
                    [7.042236, 53.350551],
                ],
            },
        },
    ],
}


@pytest.fixture
def bootstrap(map, live_server):
    map.settings["properties"]["onLoadPanel"] = "databrowser"
    map.save()
    DataLayerFactory(map=map, data=DATALAYER_DATA)


def test_umap_export(map, live_server, bootstrap, page):
    page.goto(f"{live_server.url}{map.get_absolute_url()}?share")
    link = page.get_by_role("link", name="full backup")
    expect(link).to_be_visible()
    with page.expect_download() as download_info:
        link.click()
    download = download_info.value
    assert download.suggested_filename == "umap_backup_test-map.umap"
    path = Path("/tmp/") / download.suggested_filename
    download.save_as(path)
    downloaded = json.loads(path.read_text())
    del downloaded["uri"]  # Port changes at each run
    assert downloaded == {
        "geometry": {
            "coordinates": [13.447265624999998, 48.94415123418794],
            "type": "Point",
        },
        "layers": [
            {
                "_umap_options": {
                    "browsable": True,
                    "displayOnLoad": True,
                    "name": "test datalayer",
                },
                "features": [
                    {
                        "geometry": {
                            "coordinates": [
                                [
                                    [11.25, 53.585984],
                                    [10.151367, 52.975108],
                                    [12.689209, 52.167194],
                                    [14.084473, 53.199452],
                                    [12.634277, 53.618579],
                                    [11.25, 53.585984],
                                    [11.25, 53.585984],
                                ]
                            ],
                            "type": "Polygon",
                        },
                        "id": "gyNzM",
                        "properties": {"name": "name poly"},
                        "type": "Feature",
                    },
                    {
                        "geometry": {
                            "coordinates": [-0.274658, 52.57635],
                            "type": "Point",
                        },
                        "id": "QwNjg",
                        "properties": {
                            "_umap_options": {"color": "OliveDrab"},
                            "name": "test",
                            "description": "Some description",
                        },
                        "type": "Feature",
                    },
                    {
                        "geometry": {
                            "coordinates": [
                                [-0.571289, 54.476422],
                                [0.439453, 54.610255],
                                [1.724854, 53.448807],
                                [4.163818, 53.988395],
                                [5.306396, 53.533778],
                                [6.591797, 53.709714],
                                [7.042236, 53.350551],
                            ],
                            "type": "LineString",
                        },
                        "id": "YwMTM",
                        "properties": {
                            "_umap_options": {"fill": False, "opacity": 0.6},
                            "name": "test",
                        },
                        "type": "Feature",
                    },
                ],
                "type": "FeatureCollection",
            }
        ],
        "properties": {
            "datalayersControl": True,
            "description": "Which is just the Danube, at the end",
            "displayPopupFooter": False,
            "licence": "",
            "miniMap": False,
            "moreControl": True,
            "name": "test map",
            "scaleControl": True,
            "tilelayer": {
                "attribution": "© OSM Contributors",
                "maxZoom": 18,
                "minZoom": 0,
                "url_template": "https://a.tile.openstreetmap.fr/osmfr/{z}/{x}/{y}.png",
            },
            "tilelayersControl": True,
            "zoom": 7,
            "zoomControl": True,
            "onLoadPanel": "databrowser",
        },
        "type": "umap",
    }


def test_csv_export(map, live_server, bootstrap, page):
    page.goto(f"{live_server.url}{map.get_absolute_url()}?share")
    button = page.get_by_role("button", name="csv")
    expect(button).to_be_visible()
    with page.expect_download() as download_info:
        button.click()
    download = download_info.value
    assert download.suggested_filename == "test_map.csv"
    path = Path("/tmp/") / download.suggested_filename
    download.save_as(path)
    assert (
        path.read_text()
        == """name,Latitude,Longitude,description
name poly,53.0072070131872,12.182431646910137,
test,52.57635,-0.274658,Some description
test,53.725145179688646,2.9700064980570517,"""
    )


def test_gpx_export(map, live_server, bootstrap, page):
    page.goto(f"{live_server.url}{map.get_absolute_url()}?share")
    button = page.get_by_role("button", name="gpx")
    expect(button).to_be_visible()
    with page.expect_download() as download_info:
        button.click()
    download = download_info.value
    # FIXME assert mimetype (find no way to access it throught PW)
    assert download.suggested_filename == "test_map.gpx"
    path = Path("/tmp/") / download.suggested_filename
    download.save_as(path)
    assert (
        path.read_text()
        == """<?xml version="1.0" encoding="UTF-8"?><gpx xmlns="http://www.topografix.com/GPX/1/1" version="1.1" creator="@dwayneparton/geojson-to-gpx"><wpt lat="52.57635" lon="-0.274658"><name>test</name><desc>Some description</desc></wpt><trk><name>test</name><trkseg><trkpt lat="54.476422" lon="-0.571289"/><trkpt lat="54.610255" lon="0.439453"/><trkpt lat="53.448807" lon="1.724854"/><trkpt lat="53.988395" lon="4.163818"/><trkpt lat="53.533778" lon="5.306396"/><trkpt lat="53.709714" lon="6.591797"/><trkpt lat="53.350551" lon="7.042236"/></trkseg></trk></gpx>"""
    )


def test_kml_export(map, live_server, bootstrap, page):
    page.goto(f"{live_server.url}{map.get_absolute_url()}?share")
    button = page.get_by_role("button", name="kml")
    expect(button).to_be_visible()
    with page.expect_download() as download_info:
        button.click()
    download = download_info.value
    assert download.suggested_filename == "test_map.kml"
    path = Path("/tmp/") / download.suggested_filename
    download.save_as(path)
    assert (
        path.read_text()
        == """<kml xmlns="http://www.opengis.net/kml/2.2"><Document>\n<Placemark id="gyNzM">\n<name>name poly</name><ExtendedData></ExtendedData>\n  <Polygon>\n<outerBoundaryIs>\n  <LinearRing><coordinates>11.25,53.585984\n10.151367,52.975108\n12.689209,52.167194\n14.084473,53.199452\n12.634277,53.618579\n11.25,53.585984\n11.25,53.585984</coordinates></LinearRing></outerBoundaryIs></Polygon></Placemark>\n<Placemark id="QwNjg">\n<name>test</name><description>Some description</description><ExtendedData>\n  <Data name="_umap_options"><value>{"color":"OliveDrab"}</value></Data></ExtendedData>\n  <Point><coordinates>-0.274658,52.57635</coordinates></Point></Placemark>\n<Placemark id="YwMTM">\n<name>test</name><ExtendedData>\n  <Data name="_umap_options"><value>{"fill":false,"opacity":0.6}</value></Data></ExtendedData>\n  <LineString><coordinates>-0.571289,54.476422\n0.439453,54.610255\n1.724854,53.448807\n4.163818,53.988395\n5.306396,53.533778\n6.591797,53.709714\n7.042236,53.350551</coordinates></LineString></Placemark></Document></kml>"""
    )


def test_geojson_export(map, live_server, bootstrap, page):
    page.goto(f"{live_server.url}{map.get_absolute_url()}?share")
    button = page.get_by_role("button", name="geojson")
    expect(button).to_be_visible()
    with page.expect_download() as download_info:
        button.click()
    download = download_info.value
    assert download.suggested_filename == "test_map.geojson"
    path = Path("/tmp/") / download.suggested_filename
    download.save_as(path)
    assert json.loads(path.read_text()) == {
        "features": [
            {
                "geometry": {
                    "coordinates": [
                        [
                            [11.25, 53.585984],
                            [10.151367, 52.975108],
                            [12.689209, 52.167194],
                            [14.084473, 53.199452],
                            [12.634277, 53.618579],
                            [11.25, 53.585984],
                            [11.25, 53.585984],
                        ]
                    ],
                    "type": "Polygon",
                },
                "id": "gyNzM",
                "properties": {"name": "name poly"},
                "type": "Feature",
            },
            {
                "geometry": {"coordinates": [-0.274658, 52.57635], "type": "Point"},
                "id": "QwNjg",
                "properties": {
                    "_umap_options": {"color": "OliveDrab"},
                    "name": "test",
                    "description": "Some description",
                },
                "type": "Feature",
            },
            {
                "geometry": {
                    "coordinates": [
                        [-0.571289, 54.476422],
                        [0.439453, 54.610255],
                        [1.724854, 53.448807],
                        [4.163818, 53.988395],
                        [5.306396, 53.533778],
                        [6.591797, 53.709714],
                        [7.042236, 53.350551],
                    ],
                    "type": "LineString",
                },
                "id": "YwMTM",
                "properties": {
                    "_umap_options": {"fill": False, "opacity": 0.6},
                    "name": "test",
                },
                "type": "Feature",
            },
        ],
        "type": "FeatureCollection",
    }



import os
import re

import pytest
from daphne.testing import DaphneProcess
from django.contrib.staticfiles.handlers import ASGIStaticFilesHandler
from playwright.sync_api import expect

from umap.asgi import application

from ..base import mock_tiles


@pytest.fixture(scope="session")
def browser_context_args(browser_context_args):
    return {**browser_context_args, "locale": "en-GB", "timezone_id": "Europe/Paris"}


@pytest.fixture(autouse=True)
def set_timeout(context):
    timeout = int(os.environ.get("PLAYWRIGHT_TIMEOUT", 7500))
    context.set_default_timeout(timeout)
    context.set_default_navigation_timeout(timeout)
    expect.set_options(timeout=timeout)


@pytest.fixture(autouse=True)
def mock_osm_tiles(page):
    if not bool(os.environ.get("PWDEBUG", False)):
        page.route(re.compile(r".*tile\..*"), mock_tiles)


@pytest.fixture
def new_page(context):
    def make_page(prefix="console"):
        page = context.new_page()
        page.on(
            "console",
            lambda msg: print(f"{prefix}: {msg.text}")
            if msg.type != "warning"
            else None,
        )
        page.on("pageerror", lambda exc: print(f"{prefix} uncaught exception: {exc}"))
        return page

    yield make_page


@pytest.fixture
def page(new_page):
    return new_page()


@pytest.fixture
def login(new_page, settings, live_server):
    def do_login(user, **kwargs):
        # TODO use storage state to do login only once per session
        # https://playwright.dev/python/docs/auth
        settings.ENABLE_ACCOUNT_LOGIN = True
        page = new_page(**kwargs)
        page.goto(f"{live_server.url}/en/")
        page.locator(".login").click()
        page.get_by_placeholder("Username").fill(user.username)
        page.get_by_placeholder("Password").fill("123123")
        page.locator('#login_form input[type="submit"]').click()
        return page

    return do_login


@pytest.fixture(scope="function")
def asgi_live_server(request, live_server):
    server = DaphneProcess("localhost", lambda: ASGIStaticFilesHandler(application))
    server.start()
    server.ready.wait()
    port = server.port.value
    server.url = f"http://localhost:{port}"

    yield server

    server.terminate()
    server.join()



import re

import pytest

from umap.models import Team

pytestmark = pytest.mark.django_db


def test_can_add_user_to_team(live_server, map, user, team, login):
    map.owner.teams.add(team)
    map.owner.save()
    assert Team.objects.count() == 1
    page = login(map.owner)
    with page.expect_navigation():
        page.get_by_role("link", name="My teams").click()
    with page.expect_navigation():
        page.get_by_role("link", name="Edit").click()
    page.get_by_placeholder("Add user").click()
    with page.expect_response(re.compile(r".*/agnocomplete/.*")):
        page.get_by_placeholder("Add user").press_sequentially("joe")
    page.get_by_text("Joe").click()
    page.get_by_role("button", name="Save").click()
    assert Team.objects.count() == 1
    modified = Team.objects.first()
    assert user in modified.users.all()


def test_can_remove_user_from_team(live_server, map, user, user2, team, login):
    map.owner.teams.add(team)
    map.owner.save()
    user.teams.add(team)
    user.save()
    user2.teams.add(team)
    user2.save()
    assert Team.objects.count() == 1
    page = login(map.owner)
    with page.expect_navigation():
        page.get_by_role("link", name="My teams").click()
    with page.expect_navigation():
        page.get_by_role("link", name="Edit").click()
    page.locator("li").filter(has_text="Averell").locator(".close").click()
    page.get_by_role("button", name="Save").click()
    assert Team.objects.count() == 1
    modified = Team.objects.first()
    assert user in modified.users.all()
    assert user2 not in modified.users.all()



# Generated by Django 5.1.2 on 2024-11-15 11:03

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("umap", "0022_add_team"),
    ]

    operations = [
        migrations.AlterField(
            model_name="datalayer",
            name="uuid",
            field=models.UUIDField(
                editable=False, primary_key=True, serialize=False, unique=True
            ),
        ),
    ]



import sys

from django.conf import settings
from django.core.management.base import BaseCommand
from django.utils import translation

from umap.models import Map


class Command(BaseCommand):
    help = (
        "Retrieve anonymous edit url of a map. "
        "Eg.: python manage.py anonymous_edit_url 1234"
    )

    def add_arguments(self, parser):
        parser.add_argument("pk", help="PK of the map to retrieve.")
        parser.add_argument(
            "--lang",
            help="Language code to use in the URL.",
            default=settings.LANGUAGE_CODE,
        )

    def abort(self, msg):
        self.stderr.write(msg)
        sys.exit(1)

    def handle(self, *args, **options):
        translation.activate(options["lang"])
        pk = options["pk"]
        try:
            map_ = Map.objects.get(pk=pk)
        except Map.DoesNotExist:
            self.abort("Map with pk {} not found".format(pk))
        if map_.owner:
            self.abort("Map is not anonymous (owner: {})".format(map_.owner))
        print(map_.get_anonymous_edit_url())



import re

import pytest
from playwright.sync_api import expect

pytestmark = pytest.mark.django_db


def test_iframe_code_can_contain_datalayers(map, live_server, datalayer, page):
    page.goto(f"{live_server.url}{map.get_absolute_url()}?share")
    textarea = page.locator(".umap-share-iframe")
    expect(textarea).to_be_visible()
    expect(textarea).to_have_text(re.compile('src="'))
    expect(textarea).to_have_text(re.compile('href="'))
    # We should ave both, once for iframe link, once for full screen
    expect(textarea).to_have_text(re.compile("scrollWheelZoom=true"))
    expect(textarea).to_have_text(re.compile("scrollWheelZoom=false"))
    expect(textarea).not_to_have_text(re.compile(f"datalayers={datalayer.pk}"))
    # Open options
    page.get_by_text("Embed and link options").click()
    page.get_by_title("Keep current visible layers").click()
    expect(textarea).to_have_text(re.compile(f"datalayers={datalayer.pk}"))
    # Now click again
    page.get_by_title("Keep current visible layers").click()
    expect(textarea).not_to_have_text(re.compile(f"datalayers={datalayer.pk}"))


def test_iframe_code_can_contain_feature(map, live_server, datalayer, page):
    page.goto(f"{live_server.url}{map.get_absolute_url()}?share")
    page.locator(".icon_container").click()
    textarea = page.locator(".umap-share-iframe")
    expect(textarea).to_be_visible()
    expect(textarea).not_to_have_text(re.compile("feature=Here"))
    # Open options
    page.get_by_text("Embed and link options").click()
    page.get_by_title("Open current feature on load").click()
    expect(textarea).to_have_text(re.compile("feature=Here"))
    # Click again to deactivate it
    page.get_by_title("Open current feature on load").click()
    expect(textarea).not_to_have_text(re.compile("feature=Here"))



from pathlib import Path

from django.core.files import File
from django.core.management.base import BaseCommand

from umap.models import Pictogram


class Command(BaseCommand):
    help = "Import pictograms from a folder"

    def add_arguments(self, parser):
        parser.add_argument("path")
        parser.add_argument(
            "--attribution",
            required=True,
            help="Attribution of the imported pictograms",
        )
        parser.add_argument(
            "--extensions",
            help="Optional list of extensins to process",
            nargs="+",
            default=[".svg"],
        )
        parser.add_argument(
            "--exclude",
            help="Optional list of files or dirs to exclude",
            nargs="+",
            default=["font"],
        )
        parser.add_argument(
            "--force", action="store_true", help="Update picto if it already exists."
        )

    def handle(self, *args, **options):
        self.path = Path(options["path"])
        self.attribution = options["attribution"]
        self.extensions = options["extensions"]
        self.force = options["force"]
        self.exclude = options["exclude"]
        self.handle_directory(self.path)

    def handle_directory(self, path):
        for filename in path.iterdir():
            if filename.name in self.exclude:
                continue
            if filename.is_dir():
                self.handle_directory(filename)
                continue
            if filename.suffix in self.extensions:
                name = filename.stem
                picto = Pictogram.objects.filter(name=name).last()
                if picto:
                    if not self.force:
                        self.stdout.write(
                            f"⚠ Pictogram with name '{name}' already exists. Skipping."
                        )
                        continue
                else:
                    picto = Pictogram()
                    picto.name = name
                if path.name != self.path.name:  # Subfolders only
                    picto.category = path.name
                picto.attribution = self.attribution
                with (filename).open("rb") as f:
                    picto.pictogram.save(filename.name, File(f), save=True)
                self.stdout.write(f"✔ Imported pictogram {filename}.")



from django.conf import settings
from django.core.management.base import BaseCommand

from umap.models import DataLayer
from umap.storage.fs import FSDataStorage


class Command(BaseCommand):
    help = "Migrate latest datalayers from filesystem to S3."

    def handle(self, *args, **options):
        assert settings.UMAP_READONLY, "You must run that script with a read-only uMap."
        assert (
            settings.STORAGES["data"]["BACKEND"] == "umap.storage.s3.S3DataStorage"
        ), "You must configure your storages to point to S3"
        fs_storage = FSDataStorage()
        for datalayer in DataLayer.objects.all():
            geojson_fs_path = str(datalayer.geojson)
            try:
                datalayer.geojson.save(
                    datalayer.geojson.name, fs_storage.open(geojson_fs_path)
                )
            except FileNotFoundError as e:
                print(e)
                print(geojson_fs_path, "not found on disk")
                continue
            if options["verbosity"] > 1:
                print("Migrated:", geojson_fs_path)
            datalayer.save(force_update=True)



import json
import zipfile
from io import BytesIO

import pytest
from django.contrib.auth import get_user_model
from django.core import mail
from django.core.signing import Signer
from django.urls import reverse
from django.utils import translation

from umap.models import DataLayer, Map, Star

from .base import MapFactory, UserFactory, login_required

pytestmark = pytest.mark.django_db
User = get_user_model()


@pytest.fixture
def post_data():
    return {
        "name": "name",
        "center": '{"type":"Point","coordinates":[13.447265624999998,48.94415123418794]}',  # noqa
        "settings": '{"type":"Feature","geometry":{"type":"Point","coordinates":[5.0592041015625,52.05924589011585]},"properties":{"tilelayer":{"maxZoom":20,"url_template":"http://a.tile.openstreetmap.fr/hot/{z}/{x}/{y}.png","minZoom":0,"attribution":"HOT and friends"},"licence":"","description":"","name":"test enrhûmé","tilelayersControl":true,"displayDataBrowserOnLoad":false,"displayPopupFooter":true,"displayCaptionOnLoad":false,"miniMap":true,"moreControl":true,"scaleControl":true,"zoomControl":true,"datalayersControl":true,"zoom":8}}',  # noqa
    }


def test_create(client, user, post_data):
    url = reverse("map_create")
    # POST only mendatory fields
    name = "test-map-with-new-name"
    post_data["name"] = name
    client.login(username=user.username, password="123123")
    response = client.post(url, post_data)
    assert response.status_code == 200
    j = json.loads(response.content.decode())
    created_map = Map.objects.latest("pk")
    assert j["id"] == created_map.pk
    assert created_map.name == name
    assert created_map.center.x == 13.447265624999998
    assert created_map.center.y == 48.94415123418794
    assert j["permissions"] == {
        "edit_status": 3,
        "share_status": 0,
        "owner": {"id": user.pk, "name": "Joe", "url": "/en/user/Joe/"},
        "editors": [],
    }


def test_map_create_permissions(client, settings):
    settings.UMAP_ALLOW_ANONYMOUS = False
    url = reverse("map_create")
    # POST anonymous
    response = client.post(url, {})
    assert login_required(response)


def test_map_update_access(client, map, user):
    url = reverse("map_update", kwargs={"map_id": map.pk})
    # GET anonymous
    response = client.get(url)
    assert login_required(response)
    # POST anonymous
    response = client.post(url, {})
    assert login_required(response)
    # GET with wrong permissions
    client.login(username=user.username, password="123123")
    response = client.get(url)
    assert response.status_code == 403
    # POST with wrong permissions
    client.login(username=user.username, password="123123")
    response = client.post(url, {})
    assert response.status_code == 403


def test_map_update_permissions_access(client, map, user):
    url = reverse("map_update_permissions", kwargs={"map_id": map.pk})
    # GET anonymous
    response = client.get(url)
    assert login_required(response)
    # POST anonymous
    response = client.post(url, {})
    assert login_required(response)
    # GET with wrong permissions
    client.login(username=user.username, password="123123")
    response = client.get(url)
    assert response.status_code == 403
    # POST with wrong permissions
    client.login(username=user.username, password="123123")
    response = client.post(url, {})
    assert response.status_code == 403


def test_update(client, map, post_data):
    url = reverse("map_update", kwargs={"map_id": map.pk})
    # POST only mendatory fields
    name = "new map name"
    post_data["name"] = name
    client.login(username=map.owner.username, password="123123")
    response = client.post(url, post_data)
    assert response.status_code == 200
    j = json.loads(response.content.decode())
    assert "html" not in j
    updated_map = Map.objects.get(pk=map.pk)
    assert j["id"] == updated_map.pk
    assert updated_map.name == name


def test_delete(client, map, datalayer):
    url = reverse("map_delete", args=(map.pk,))
    client.login(username=map.owner.username, password="123123")
    response = client.post(
        url, headers={"X-Requested-With": "XMLHttpRequest"}, follow=True
    )
    assert response.status_code == 200
    assert Map.objects.filter(pk=map.pk).exists()
    assert DataLayer.objects.filter(pk=datalayer.pk).exists()
    reloaded = Map.objects.get(pk=map.pk)
    assert reloaded.share_status == Map.DELETED
    # Check that user has not been impacted
    assert User.objects.filter(pk=map.owner.pk).exists()
    # Test response is a json
    j = json.loads(response.content.decode())
    assert "redirect" in j


def test_wrong_slug_should_redirect_to_canonical(client, map):
    url = reverse("map", kwargs={"map_id": map.pk, "slug": "wrong-slug"})
    canonical = reverse("map", kwargs={"map_id": map.pk, "slug": map.slug})
    response = client.get(url)
    assert response.status_code == 301
    assert response["Location"] == canonical


def test_wrong_slug_should_redirect_with_query_string(client, map):
    url = reverse("map", kwargs={"map_id": map.pk, "slug": "wrong-slug"})
    url = "{}?editMode=simple".format(url)
    canonical = reverse("map", kwargs={"map_id": map.pk, "slug": map.slug})
    canonical = "{}?editMode=simple".format(canonical)
    response = client.get(url)
    assert response.status_code == 301
    assert response["Location"] == canonical


def test_should_not_consider_the_query_string_for_canonical_check(client, map):
    url = reverse("map", kwargs={"map_id": map.pk, "slug": map.slug})
    url = "{}?editMode=simple".format(url)
    response = client.get(url)
    assert response.status_code == 200


def test_map_headers(client, map):
    url = reverse("map", kwargs={"map_id": map.pk, "slug": map.slug})
    response = client.get(url)
    assert response.status_code == 200
    assert response.headers["Access-Control-Allow-Origin"] == "*"


def test_short_url_should_redirect_to_canonical(client, map):
    url = reverse("map_short_url", kwargs={"pk": map.pk})
    canonical = reverse("map", kwargs={"map_id": map.pk, "slug": map.slug})
    response = client.get(url)
    assert response.status_code == 301
    assert response["Location"] == canonical


def test_clone_map_should_create_a_new_instance(client, map):
    assert Map.objects.count() == 1
    url = reverse("map_clone", kwargs={"map_id": map.pk})
    client.login(username=map.owner.username, password="123123")
    response = client.post(url)
    assert response.status_code == 302
    assert Map.objects.count() == 2
    clone = Map.objects.latest("pk")
    assert response["Location"] == clone.get_absolute_url()
    assert clone.pk != map.pk
    assert clone.name == "Clone of " + map.name


def test_clone_map_should_be_possible_via_ajax(client, map):
    assert Map.objects.count() == 1
    url = reverse("map_clone", kwargs={"map_id": map.pk})
    client.login(username=map.owner.username, password="123123")
    response = client.post(url, headers={"X-Requested-With": "XMLHttpRequest"})
    assert response.status_code == 200
    assert Map.objects.count() == 2
    clone = Map.objects.latest("pk")
    assert response.json() == {"redirect": clone.get_absolute_url()}
    assert clone.pk != map.pk
    assert clone.name == "Clone of " + map.name


def test_user_not_allowed_should_not_clone_map(client, map, user, settings):
    settings.UMAP_ALLOW_ANONYMOUS = False
    assert Map.objects.count() == 1
    url = reverse("map_clone", kwargs={"map_id": map.pk})
    map.edit_status = map.OWNER
    map.save()
    response = client.post(url)
    assert login_required(response)
    client.login(username=user.username, password="123123")
    response = client.post(url)
    assert response.status_code == 403
    map.edit_status = map.ANONYMOUS
    map.save()
    client.logout()
    response = client.post(url)
    assert response.status_code == 403
    assert Map.objects.count() == 1


def test_clone_should_set_cloner_as_owner(client, map, user):
    url = reverse("map_clone", kwargs={"map_id": map.pk})
    map.edit_status = map.COLLABORATORS
    map.editors.add(user)
    map.save()
    client.login(username=user.username, password="123123")
    response = client.post(url)
    assert response.status_code == 302
    assert Map.objects.count() == 2
    clone = Map.objects.latest("pk")
    assert clone.pk != map.pk
    assert clone.name == "Clone of " + map.name
    assert clone.owner == user


def test_map_creation_should_allow_unicode_names(client, map, post_data):
    url = reverse("map_create")
    # POST only mendatory fields
    name = "Академический"
    post_data["name"] = name
    client.login(username=map.owner.username, password="123123")
    response = client.post(url, post_data)
    assert response.status_code == 200
    j = json.loads(response.content.decode())
    created_map = Map.objects.latest("pk")
    assert j["id"] == created_map.pk
    assert created_map.name == name
    # Lower case of the russian original name
    # self.assertEqual(created_map.slug, u"академический")
    # for now we fallback to "map", see unicode_name branch
    assert created_map.slug == "map"


@pytest.mark.parametrize("share_status", [Map.PUBLIC, Map.OPEN])
def test_anonymous_can_access_map_with_share_status_accessible(
    client, map, share_status
):
    url = reverse("map", args=(map.slug, map.pk))
    map.share_status = share_status
    map.save()
    response = client.get(url)
    assert response.status_code == 200


@pytest.mark.parametrize(
    "share_status", [Map.PRIVATE, Map.DRAFT, Map.BLOCKED, Map.DELETED]
)
def test_anonymous_cannot_access_map_with_share_status_restricted(
    client, map, share_status
):
    url = reverse("map", args=(map.slug, map.pk))
    map.share_status = share_status
    map.save()
    response = client.get(url)
    assert response.status_code == 403


@pytest.mark.parametrize("share_status", [Map.PRIVATE, Map.DRAFT])
def test_owner_can_access_map_with_share_status_restricted(client, map, share_status):
    url = reverse("map", args=(map.slug, map.pk))
    map.share_status = share_status
    map.save()
    client.login(username=map.owner.username, password="123123")
    response = client.get(url)
    assert response.status_code == 200


@pytest.mark.parametrize("share_status", [Map.PRIVATE, Map.DRAFT])
def test_editors_can_access_map_with_share_status_resricted(
    client, map, user, share_status
):
    url = reverse("map", args=(map.slug, map.pk))
    map.share_status = share_status
    map.editors.add(user)
    map.save()
    client.login(username=user.username, password="123123")
    response = client.get(url)
    assert response.status_code == 200


def test_owner_cannot_access_map_with_share_status_deleted(client, map):
    url = reverse("map", args=(map.slug, map.pk))
    map.share_status = map.DELETED
    map.save()
    client.login(username=map.owner.username, password="123123")
    response = client.get(url)
    assert response.status_code == 403


def test_owner_cannot_access_map_with_share_status_blocked(client, map):
    url = reverse("map", args=(map.slug, map.pk))
    map.share_status = map.BLOCKED
    map.save()
    client.login(username=map.owner.username, password="123123")
    response = client.get(url)
    assert response.status_code == 403


def test_non_editor_cannot_access_map_if_share_status_private(client, map, user):
    url = reverse("map", args=(map.slug, map.pk))
    map.share_status = map.PRIVATE
    map.save()
    client.login(username=user.username, password="123123")
    response = client.get(url)
    assert response.status_code == 403


def test_map_geojson_view(client, map):
    url = reverse("map_geojson", args=(map.pk,))
    response = client.get(url)
    j = json.loads(response.content.decode())
    assert "json" in response["content-type"]
    assert "type" in j


def test_only_owner_can_delete(client, map, user):
    map.editors.add(user)
    url = reverse("map_delete", kwargs={"map_id": map.pk})
    client.login(username=user.username, password="123123")
    response = client.post(
        url, headers={"X-Requested-With": "XMLHttpRequest"}, follow=True
    )
    assert response.status_code == 403


def test_map_editors_do_not_see_owner_change_input(client, map, user):
    map.editors.add(user)
    map.edit_status = map.COLLABORATORS
    map.save()
    url = reverse("map_update_permissions", kwargs={"map_id": map.pk})
    client.login(username=user.username, password="123123")
    response = client.get(url)
    assert "id_owner" not in response


def test_logged_in_user_can_edit_map_editable_by_anonymous(client, map, user):
    map.owner = None
    map.edit_status = map.ANONYMOUS
    map.save()
    client.login(username=user.username, password="123123")
    url = reverse("map_update", kwargs={"map_id": map.pk})
    new_name = "this is my new name"
    data = {
        "center": '{"type":"Point","coordinates":[13.447265624999998,48.94415123418794]}',  # noqa
        "name": new_name,
    }
    response = client.post(url, data)
    assert response.status_code == 200
    assert Map.objects.get(pk=map.pk).name == new_name


@pytest.mark.usefixtures("allow_anonymous")
def test_anonymous_create(cookieclient, post_data):
    url = reverse("map_create")
    # POST only mendatory fields
    name = "test-map-with-new-name"
    post_data["name"] = name
    response = cookieclient.post(url, post_data)
    assert response.status_code == 200
    j = json.loads(response.content.decode())
    created_map = Map.objects.latest("pk")
    assert j["id"] == created_map.pk
    assert (
        created_map.get_anonymous_edit_url() in j["permissions"]["anonymous_edit_url"]
    )
    assert j["user"]["is_owner"] is True
    assert created_map.name == name
    key, value = created_map.signed_cookie_elements
    assert key in cookieclient.cookies


@pytest.mark.usefixtures("allow_anonymous")
def test_anonymous_update_without_cookie_fails(client, anonymap, post_data):
    url = reverse("map_update", kwargs={"map_id": anonymap.pk})
    response = client.post(url, post_data)
    assert response.status_code == 403


@pytest.mark.usefixtures("allow_anonymous")
def test_anonymous_update_with_cookie_should_work(cookieclient, anonymap, post_data):
    url = reverse("map_update", kwargs={"map_id": anonymap.pk})
    # POST only mendatory fields
    name = "new map name"
    post_data["name"] = name
    response = cookieclient.post(url, post_data)
    assert response.status_code == 200
    j = json.loads(response.content.decode())
    updated_map = Map.objects.get(pk=anonymap.pk)
    assert j["id"] == updated_map.pk


@pytest.mark.usefixtures("allow_anonymous")
def test_anonymous_delete(cookieclient, anonymap):
    url = reverse("map_delete", args=(anonymap.pk,))
    response = cookieclient.post(
        url, headers={"X-Requested-With": "XMLHttpRequest"}, follow=True
    )
    assert response.status_code == 200
    assert Map.objects.filter(pk=anonymap.pk).exists()
    reloaded = Map.objects.get(pk=anonymap.pk)
    assert reloaded.share_status == Map.DELETED
    # Test response is a json
    j = json.loads(response.content.decode())
    assert "redirect" in j


@pytest.mark.usefixtures("allow_anonymous")
def test_no_cookie_cannot_delete(client, anonymap):
    url = reverse("map_delete", args=(anonymap.pk,))
    response = client.post(
        url, headers={"X-Requested-With": "XMLHttpRequest"}, follow=True
    )
    assert response.status_code == 403


@pytest.mark.usefixtures("allow_anonymous")
def test_no_cookie_cannot_view_anonymous_owned_map_in_draft(client, anonymap):
    anonymap.share_status = Map.DRAFT
    anonymap.save()
    url = reverse("map", kwargs={"map_id": anonymap.pk, "slug": anonymap.slug})
    response = client.get(url)
    assert response.status_code == 403


@pytest.mark.usefixtures("allow_anonymous")
def test_owner_can_view_anonymous_owned_map_in_draft(cookieclient, anonymap):
    anonymap.share_status = Map.DRAFT
    anonymap.save()
    url = reverse("map", kwargs={"map_id": anonymap.pk, "slug": anonymap.slug})
    response = cookieclient.get(url)
    assert response.status_code == 200


@pytest.mark.usefixtures("allow_anonymous")
def test_anonymous_edit_url(cookieclient, anonymap):
    url = anonymap.get_anonymous_edit_url()
    canonical = reverse("map", kwargs={"map_id": anonymap.pk, "slug": anonymap.slug})
    response = cookieclient.get(url)
    assert response.status_code == 302
    assert response["Location"] == canonical
    key, value = anonymap.signed_cookie_elements
    assert key in cookieclient.cookies


@pytest.mark.usefixtures("allow_anonymous")
def test_sha1_anonymous_edit_url(cookieclient, anonymap):
    signer = Signer(algorithm="sha1")
    signature = signer.sign(anonymap.pk)
    url = reverse("map_anonymous_edit_url", kwargs={"signature": signature})
    canonical = reverse("map", kwargs={"map_id": anonymap.pk, "slug": anonymap.slug})
    response = cookieclient.get(url)
    assert response.status_code == 302
    assert response["Location"] == canonical
    key, value = anonymap.signed_cookie_elements
    assert key in cookieclient.cookies


@pytest.mark.usefixtures("allow_anonymous")
def test_bad_anonymous_edit_url_should_return_403(cookieclient, anonymap):
    url = anonymap.get_anonymous_edit_url()
    url = reverse(
        "map_anonymous_edit_url", kwargs={"signature": "%s:badsignature" % anonymap.pk}
    )
    response = cookieclient.get(url)
    assert response.status_code == 403


@pytest.mark.usefixtures("allow_anonymous")
def test_clone_anonymous_map_should_not_be_possible_if_user_is_not_allowed(
    client, anonymap, user
):
    assert Map.objects.count() == 1
    url = reverse("map_clone", kwargs={"map_id": anonymap.pk})
    anonymap.edit_status = anonymap.OWNER
    anonymap.save()
    response = client.post(url)
    assert response.status_code == 403
    client.login(username=user.username, password="123123")
    response = client.post(url)
    assert response.status_code == 403
    assert Map.objects.count() == 1


@pytest.mark.usefixtures("allow_anonymous")
def test_clone_map_should_be_possible_if_edit_status_is_anonymous(client, anonymap):
    assert Map.objects.count() == 1
    url = reverse("map_clone", kwargs={"map_id": anonymap.pk})
    anonymap.edit_status = anonymap.ANONYMOUS
    anonymap.save()
    response = client.post(url)
    assert response.status_code == 302
    assert Map.objects.count() == 2
    clone = Map.objects.latest("pk")
    assert response["Location"] == clone.get_absolute_url()
    assert clone.pk != anonymap.pk
    assert clone.name == "Clone of " + anonymap.name
    assert clone.owner is None


@pytest.mark.usefixtures("allow_anonymous")
def test_anyone_can_access_anonymous_map(cookieclient, anonymap):
    url = reverse("map", args=(anonymap.slug, anonymap.pk))
    anonymap.share_status = anonymap.PUBLIC
    response = cookieclient.get(url)
    assert response.status_code == 200
    anonymap.share_status = anonymap.OPEN
    response = cookieclient.get(url)
    assert response.status_code == 200
    anonymap.share_status = anonymap.PRIVATE
    response = cookieclient.get(url)
    assert response.status_code == 200


@pytest.mark.usefixtures("allow_anonymous")
def test_map_attach_owner(cookieclient, anonymap, user):
    url = reverse("map_attach_owner", kwargs={"map_id": anonymap.pk})
    cookieclient.login(username=user.username, password="123123")
    assert anonymap.owner is None
    response = cookieclient.post(url)
    assert response.status_code == 200
    map = Map.objects.get(pk=anonymap.pk)
    assert map.owner == user


@pytest.mark.usefixtures("allow_anonymous")
def test_map_attach_owner_not_logged_in(cookieclient, anonymap, user):
    url = reverse("map_attach_owner", kwargs={"map_id": anonymap.pk})
    assert anonymap.owner is None
    response = cookieclient.post(url)
    assert response.status_code == 403


@pytest.mark.usefixtures("allow_anonymous")
def test_map_attach_owner_with_already_an_owner(cookieclient, map, user):
    url = reverse("map_attach_owner", kwargs={"map_id": map.pk})
    cookieclient.login(username=user.username, password="123123")
    assert map.owner
    assert map.owner != user
    response = cookieclient.post(url)
    assert response.status_code == 403


def test_map_attach_owner_anonymous_not_allowed(cookieclient, anonymap, user):
    url = reverse("map_attach_owner", kwargs={"map_id": anonymap.pk})
    cookieclient.login(username=user.username, password="123123")
    assert anonymap.owner is None
    response = cookieclient.post(url)
    assert response.status_code == 403

    # # GET anonymous
    # response = client.get(url)
    # assert login_required(response)
    # # POST anonymous
    # response = client.post(url, {})
    # assert login_required(response)
    # # GET with wrong permissions
    # client.login(username=user.username, password="123123")
    # response = client.get(url)
    # assert response.status_code == 403
    # # POST with wrong permissions
    # client.login(username=user.username, password="123123")
    # response = client.post(url, {})
    # assert response.status_code == 403


def test_create_readonly(client, user, post_data, settings):
    settings.UMAP_READONLY = True
    url = reverse("map_create")
    client.login(username=user.username, password="123123")
    response = client.post(url, post_data)
    assert response.status_code == 403
    assert response.content == b"Site is readonly for maintenance"


def test_authenticated_user_can_star_map(client, map, user):
    url = reverse("map_star", args=(map.pk,))
    client.login(username=user.username, password="123123")
    assert Star.objects.filter(by=user).count() == 0
    response = client.post(url)
    assert response.status_code == 200
    assert Star.objects.filter(by=user).count() == 1


def test_anonymous_cannot_star_map(client, map):
    url = reverse("map_star", args=(map.pk,))
    assert Star.objects.count() == 0
    response = client.post(url)
    assert response.status_code == 302
    assert "login" in response["Location"]
    assert Star.objects.count() == 0


def test_user_can_see_their_star(client, map, user):
    url = reverse("map_star", args=(map.pk,))
    client.login(username=user.username, password="123123")
    assert Star.objects.filter(by=user).count() == 0
    response = client.post(url)
    assert response.status_code == 200
    url = reverse("user_stars", args=(user.username,))
    response = client.get(url)
    assert response.status_code == 200
    assert map.name in response.content.decode()


def test_stars_link(client, map, user):
    client.login(username=user.username, password="123123")
    response = client.get(reverse("home"))
    assert response.status_code == 200
    assert "/user/Joe/stars" in response.content.decode()


@pytest.mark.usefixtures("allow_anonymous")
def test_cannot_send_link_on_owned_map(client, map):
    assert len(mail.outbox) == 0
    url = reverse("map_send_edit_link", args=(map.pk,))
    resp = client.post(url, {"email": "foo@bar.org"})
    assert resp.status_code == 200
    assert json.loads(resp.content.decode()) == {"login_required": "/en/login/"}
    assert len(mail.outbox) == 0


@pytest.mark.usefixtures("allow_anonymous")
def test_cannot_send_link_on_anonymous_map_without_cookie(client, anonymap):
    assert len(mail.outbox) == 0
    url = reverse("map_send_edit_link", args=(anonymap.pk,))
    resp = client.post(url, {"email": "foo@bar.org"})
    assert resp.status_code == 403
    assert len(mail.outbox) == 0


@pytest.mark.usefixtures("allow_anonymous")
def test_can_send_link_on_anonymous_map_with_cookie(cookieclient, anonymap):
    assert len(mail.outbox) == 0
    url = reverse("map_send_edit_link", args=(anonymap.pk,))
    resp = cookieclient.post(url, {"email": "foo@bar.org"})
    assert resp.status_code == 200
    assert len(mail.outbox) == 1
    assert mail.outbox[0].subject == "The uMap edit link for your map: test map"


def test_download(client, map, datalayer):
    url = reverse("map_download", args=(map.pk,))
    response = client.get(url)
    assert response.status_code == 200
    # Test response is a json
    j = json.loads(response.content.decode())
    assert j["type"] == "umap"
    assert j["uri"] == f"http://testserver/en/map/test-map_{map.pk}"
    assert j["geometry"] == {
        "coordinates": [13.447265624999998, 48.94415123418794],
        "type": "Point",
    }
    assert j["properties"] == {
        "datalayersControl": True,
        "description": "Which is just the Danube, at the end",
        "displayPopupFooter": False,
        "licence": "",
        "miniMap": False,
        "moreControl": True,
        "name": "test map",
        "scaleControl": True,
        "tilelayer": {
            "attribution": "© OSM Contributors",
            "maxZoom": 18,
            "minZoom": 0,
            "url_template": "https://a.tile.openstreetmap.fr/osmfr/{z}/{x}/{y}.png",
        },
        "tilelayersControl": True,
        "zoom": 7,
        "zoomControl": True,
    }
    assert j["layers"] == [
        {
            "_umap_options": {
                "browsable": True,
                "displayOnLoad": True,
                "name": "test datalayer",
            },
            "features": [
                {
                    "geometry": {
                        "coordinates": [14.68896484375, 48.55297816440071],
                        "type": "Point",
                    },
                    "id": "ExNTQ",
                    "properties": {
                        "_umap_options": {"color": "DarkCyan", "iconClass": "Ball"},
                        "description": "Da place anonymous again 755",
                        "name": "Here",
                    },
                    "type": "Feature",
                }
            ],
            "type": "FeatureCollection",
        },
    ]


def test_download_multiple_maps(client, map, datalayer):
    map.share_status = Map.PRIVATE
    map.save()
    another_map = MapFactory(
        owner=map.owner, name="Another map", share_status=Map.PUBLIC
    )
    client.login(username=map.owner.username, password="123123")
    url = reverse("user_download")
    response = client.get(f"{url}?map_id={map.id}&map_id={another_map.id}")
    assert response.status_code == 200
    with zipfile.ZipFile(file=BytesIO(response.content), mode="r") as f:
        assert len(f.infolist()) == 2
        assert f.infolist()[0].filename == f"umap_backup_test-map_{another_map.id}.umap"
        assert f.infolist()[1].filename == f"umap_backup_test-map_{map.id}.umap"
        with f.open(f.infolist()[1]) as umap_file:
            umapjson = json.loads(umap_file.read().decode())
            assert list(umapjson.keys()) == [
                "type",
                "geometry",
                "properties",
                "uri",
                "layers",
            ]
            assert umapjson["type"] == "umap"
            assert umapjson["uri"] == f"http://testserver/en/map/test-map_{map.id}"


def test_download_multiple_maps_unauthorized(client, map, datalayer):
    map.share_status = Map.PRIVATE
    map.save()
    user1 = UserFactory(username="user1")
    another_map = MapFactory(owner=user1, name="Another map", share_status=Map.PUBLIC)
    client.login(username=map.owner.username, password="123123")
    url = reverse("user_download")
    response = client.get(f"{url}?map_id={map.id}&map_id={another_map.id}")
    assert response.status_code == 200
    with zipfile.ZipFile(file=BytesIO(response.content), mode="r") as f:
        assert len(f.infolist()) == 1
        assert f.infolist()[0].filename == f"umap_backup_test-map_{map.id}.umap"


def test_download_multiple_maps_editor(client, map, datalayer):
    map.share_status = Map.PRIVATE
    map.save()
    user1 = UserFactory(username="user1")
    another_map = MapFactory(owner=user1, name="Another map", share_status=Map.PUBLIC)
    another_map.editors.add(map.owner)
    another_map.save()
    client.login(username=map.owner.username, password="123123")
    url = reverse("user_download")
    response = client.get(f"{url}?map_id={map.id}&map_id={another_map.id}")
    assert response.status_code == 200
    with zipfile.ZipFile(file=BytesIO(response.content), mode="r") as f:
        assert len(f.infolist()) == 2
        assert f.infolist()[0].filename == f"umap_backup_test-map_{another_map.id}.umap"
        assert f.infolist()[1].filename == f"umap_backup_test-map_{map.id}.umap"


@pytest.mark.parametrize(
    "share_status", [Map.PRIVATE, Map.BLOCKED, Map.DRAFT, Map.DELETED]
)
def test_download_shared_status_map(client, map, datalayer, share_status):
    map.share_status = share_status
    map.save()
    url = reverse("map_download", args=(map.pk,))
    response = client.get(url)
    assert response.status_code == 403


@pytest.mark.parametrize("share_status", [Map.PRIVATE, Map.DRAFT])
def test_download_my_map(client, map, datalayer, share_status):
    map.share_status = share_status
    map.save()
    client.login(username=map.owner.username, password="123123")
    url = reverse("map_download", args=(map.pk,))
    response = client.get(url)
    assert response.status_code == 200
    # Test response is a json
    j = json.loads(response.content.decode())
    assert j["type"] == "umap"


@pytest.mark.parametrize("share_status", [Map.BLOCKED, Map.DELETED])
def test_download_my_map_blocked_or_deleted(client, map, datalayer, share_status):
    map.share_status = share_status
    map.save()
    client.login(username=map.owner.username, password="123123")
    url = reverse("map_download", args=(map.pk,))
    response = client.get(url)
    assert response.status_code == 403


@pytest.mark.parametrize(
    "share_status", [Map.PRIVATE, Map.BLOCKED, Map.OPEN, Map.DRAFT]
)
def test_oembed_shared_status_map(client, map, datalayer, share_status):
    map.share_status = share_status
    map.save()
    url = f"{reverse('map_oembed')}?url=http://testserver{map.get_absolute_url()}"
    response = client.get(url)
    assert response.status_code == 403


def test_download_does_not_include_delete_datalayers(client, map, datalayer):
    datalayer.share_status = DataLayer.DELETED
    datalayer.save()
    url = reverse("map_download", args=(map.pk,))
    response = client.get(url)
    assert response.status_code == 200
    # Test response is a json
    j = json.loads(response.content.decode())
    assert j["layers"] == []


def test_oembed_no_url_map(client, map, datalayer):
    url = reverse("map_oembed")
    response = client.get(url)
    assert response.status_code == 404


def test_oembed_unknown_url_map(client, map, datalayer):
    map_url = f"http://testserver{map.get_absolute_url()}"
    # We change to an unknown id prefix to keep URL structure.
    map_url = map_url.replace("map_", "_111")
    url = f"{reverse('map_oembed')}?url={map_url}"
    response = client.get(url)
    assert response.status_code == 404


def test_oembed_wrong_format_map(client, map, datalayer):
    url = (
        f"{reverse('map_oembed')}"
        f"?url=http://testserver{map.get_absolute_url()}&format=xml"
    )
    response = client.get(url)
    assert response.status_code == 501


def test_oembed_wrong_domain_map(client, map, datalayer):
    url = f"{reverse('map_oembed')}?url=http://BADserver{map.get_absolute_url()}"
    response = client.get(url)
    assert response.status_code == 404


def test_oembed_map(client, map, datalayer):
    url = f"{reverse('map_oembed')}?url=http://testserver{map.get_absolute_url()}"
    response = client.get(url)
    assert response.status_code == 200
    assert response.headers["Access-Control-Allow-Origin"] == "*"
    j = json.loads(response.content.decode())
    assert j["type"] == "rich"
    assert j["version"] == "1.0"
    assert j["width"] == 800
    assert j["height"] == 300
    assert j["html"] == (
        '<iframe width="100%" height="300px" frameborder="0" allowfullscreen '
        f'allow="geolocation" src="//testserver/en/map/test-map_{map.id}"></iframe>'
        f'<p><a href="//testserver/en/map/test-map_{map.id}">See full screen</a></p>'
    )


def test_oembed_map_with_non_default_language(client, map, datalayer):
    translation.activate("en")
    path = map.get_absolute_url()
    assert path.startswith("/en/")
    path = path.replace("/en/", "/fr/")
    url = f"{reverse('map_oembed')}?url=http://testserver{path}"
    response = client.get(url)
    assert response.status_code == 200
    translation.activate("en")


def test_oembed_link(client, map, datalayer):
    response = client.get(map.get_absolute_url())
    assert response.status_code == 200

    assert (
        '<link rel="alternate"\n        type="application/json+oembed"'
    ) in response.content.decode()
    assert (
        'href="http://testserver/map/oembed/'
        f'?url=http%3A%2F%2Ftestserver%2Fen%2Fmap%2Ftest-map_{map.id}&format=json"'
    ) in response.content.decode()
    assert 'title="test map oEmbed URL" />' in response.content.decode()


def test_ogp_links(client, map, datalayer):
    response = client.get(map.get_absolute_url())
    assert response.status_code == 200
    content = response.content.decode()
    assert (
        f'<meta property="og:url" content="http://umap.org{map.get_absolute_url()}" />'
        in content
    )
    assert f'<meta property="og:title" content="{map.name}" />' in content
    assert f'<meta property="og:description" content="{map.description}" />' in content
    assert '<meta property="og:site_name" content="uMap" />' in content


def test_non_public_map_should_have_noindex_meta(client, map, datalayer):
    map.share_status = Map.OPEN
    map.save()
    response = client.get(map.get_absolute_url())
    assert response.status_code == 200
    assert (
        '<meta name="robots" content="noindex,nofollow">' in response.content.decode()
    )


def test_demo_instance_should_have_noindex(client, map, datalayer, settings):
    settings.UMAP_DEMO_SITE = True
    response = client.get(map.get_absolute_url())
    assert response.status_code == 200
    assert (
        '<meta name="robots" content="noindex,nofollow">' in response.content.decode()
    )



import re
from smtplib import SMTPException
from unittest.mock import patch

import pytest
from django.core.signing import get_cookie_signer
from playwright.sync_api import expect

from umap.models import DataLayer, Map

from ..base import DataLayerFactory

pytestmark = [pytest.mark.django_db, pytest.mark.usefixtures("allow_anonymous")]


@pytest.fixture
def owner_session(anonymap, context, live_server):
    key, value = anonymap.signed_cookie_elements
    signed = get_cookie_signer(salt=key).sign(value)
    context.add_cookies([{"name": key, "value": signed, "url": live_server.url}])
    return context.new_page()


def test_map_load_with_owner(anonymap, live_server, owner_session):
    owner_session.goto(f"{live_server.url}{anonymap.get_absolute_url()}")
    map_el = owner_session.locator("#map")
    expect(map_el).to_be_visible()
    enable = owner_session.get_by_role("button", name="Edit")
    expect(enable).to_be_visible()
    enable.click()
    disable = owner_session.get_by_role("button", name="View")
    expect(disable).to_be_visible()
    save = owner_session.get_by_role("button", name="Save")
    expect(save).to_be_visible()
    add_marker = owner_session.get_by_title("Draw a marker")
    expect(add_marker).to_be_visible()
    edit_settings = owner_session.get_by_title("Map advanced properties")
    expect(edit_settings).to_be_visible()
    edit_permissions = owner_session.get_by_title("Update permissions and editors")
    expect(edit_permissions).to_be_visible()


def test_map_load_with_anonymous(anonymap, live_server, page):
    page.goto(f"{live_server.url}{anonymap.get_absolute_url()}")
    map_el = page.locator("#map")
    expect(map_el).to_be_visible()
    enable = page.get_by_role("link", name="Edit")
    expect(enable).to_be_hidden()


def test_map_load_with_anonymous_but_editable_layer(
    anonymap, live_server, page, datalayer
):
    datalayer.edit_status = DataLayer.ANONYMOUS
    datalayer.save()
    page.goto(f"{live_server.url}{anonymap.get_absolute_url()}")
    map_el = page.locator("#map")
    expect(map_el).to_be_visible()
    enable = page.get_by_role("button", name="Edit")
    expect(enable).to_be_visible()
    enable.click()
    disable = page.get_by_role("button", name="View")
    expect(disable).to_be_visible()
    save = page.get_by_role("button", name="Save")
    expect(save).to_be_visible()
    add_marker = page.get_by_title("Draw a marker")
    expect(add_marker).to_be_visible()
    edit_settings = page.get_by_title("Map advanced properties")
    expect(edit_settings).to_be_hidden()
    edit_permissions = page.get_by_title("Update permissions and editors")
    expect(edit_permissions).to_be_hidden()


def test_owner_permissions_form(map, datalayer, live_server, owner_session):
    owner_session.goto(f"{live_server.url}{map.get_absolute_url()}?edit")
    edit_permissions = owner_session.get_by_title("Update permissions and editors")
    expect(edit_permissions).to_be_visible()
    edit_permissions.click()
    owner_field = owner_session.locator(".umap-field-owner")
    expect(owner_field).to_be_hidden()
    editors_field = owner_session.locator(".umap-field-editors input")
    expect(editors_field).to_be_hidden()
    datalayer_label = owner_session.get_by_text('Who can edit "test datalayer"')
    expect(datalayer_label).to_be_visible()
    options = owner_session.locator(
        ".datalayer-permissions select[name='edit_status'] option"
    )
    expect(options).to_have_count(3)
    option = owner_session.locator(
        ".datalayer-permissions select[name='edit_status'] option:checked"
    )
    expect(option).to_have_text("Inherit")
    expect(owner_session.locator(".umap-field-share_status select")).to_be_visible()
    options = [
        int(option.get_attribute("value"))
        for option in owner_session.locator(
            ".umap-field-share_status select option"
        ).all()
    ]
    assert options == [Map.DRAFT, Map.PUBLIC]
    # This field should not be present in anonymous maps
    expect(owner_session.locator(".umap-field-owner")).to_be_hidden()


def test_anonymous_can_add_marker_on_editable_layer(
    anonymap, datalayer, live_server, page
):
    datalayer.edit_status = DataLayer.OWNER
    datalayer.name = "Should not be in the select"
    datalayer.save()  # Non editable by anonymous users
    assert datalayer.map == anonymap
    other = DataLayerFactory(
        map=anonymap, edit_status=DataLayer.ANONYMOUS, name="Editable"
    )
    assert other.map == anonymap
    page.goto(f"{live_server.url}{anonymap.get_absolute_url()}?edit")
    add_marker = page.get_by_title("Draw a marker")
    expect(add_marker).to_be_visible()
    marker = page.locator(".leaflet-marker-icon")
    map_el = page.locator("#map")
    expect(marker).to_have_count(2)
    panel = page.locator(".panel.right.on")
    expect(panel).to_be_hidden()
    add_marker.click()
    map_el.click(position={"x": 100, "y": 100})
    expect(marker).to_have_count(3)
    # Edit panel is open
    expect(panel).to_be_visible()
    datalayer_select = page.locator("select[name='datalayer']")
    expect(datalayer_select).to_be_visible()
    options = page.locator("select[name='datalayer'] option")
    expect(options).to_have_count(1)  # Only Editable layer should be listed
    option = page.locator("select[name='datalayer'] option:checked")
    expect(option).to_have_text(other.name)


def test_can_change_perms_after_create(tilelayer, live_server, page):
    page.goto(f"{live_server.url}/en/map/new")
    # Create a layer
    page.get_by_title("Manage layers").click()
    page.get_by_title("Add a layer").click()
    page.locator("input[name=name]").fill("Layer 1")
    expect(
        page.get_by_role("button", name="Visibility: Draft (private)")
    ).to_be_visible()
    expect(page.get_by_role("button", name="Save", exact=True)).to_be_hidden()
    with page.expect_response(re.compile(r".*/datalayer/create/.*")):
        page.get_by_role("button", name="Save draft", exact=True).click()
    edit_permissions = page.get_by_title("Update permissions and editors")
    expect(edit_permissions).to_be_visible()
    edit_permissions.click()
    owner_field = page.locator(".umap-field-owner")
    expect(owner_field).to_be_hidden()
    editors_field = page.locator(".umap-field-editors input")
    expect(editors_field).to_be_hidden()
    datalayer_label = page.get_by_text('Who can edit "Layer 1"')
    expect(datalayer_label).to_be_visible()
    options = page.locator(".datalayer-permissions select[name='edit_status'] option")
    expect(options).to_have_count(3)
    option = page.locator(
        ".datalayer-permissions select[name='edit_status'] option:checked"
    )
    expect(option).to_have_text("Inherit")
    expect(page.get_by_label("Secret edit link:")).to_be_visible()
    page.locator('select[name="share_status"]').select_option("1")
    expect(page.get_by_role("button", name="Save draft", exact=True)).to_be_hidden()
    expect(page.get_by_role("button", name="Save", exact=True)).to_be_visible()


def test_alert_message_after_create(
    tilelayer, live_server, page, monkeypatch, settings
):
    page.goto(f"{live_server.url}/en/map/new")
    save = page.get_by_role("button", name="Save")
    expect(save).to_be_visible()
    alert = page.locator('umap-alert-creation div[role="dialog"]')
    expect(alert).to_be_hidden()
    with page.expect_response(re.compile(r".*/map/create/")):
        save.click()
    new_map = Map.objects.last()
    expect(alert).to_be_visible()
    expect(
        alert.get_by_text("Your map has been created with an anonymous account!")
    ).to_be_visible()
    expect(alert.get_by_role("button", name="Copy")).to_be_visible()
    expect(alert.get_by_role("button", name="Send me the link")).to_be_visible()
    alert.get_by_placeholder("Email").fill("foo@bar.com")
    with patch("umap.views.send_mail") as patched:
        with page.expect_response(re.compile("/en/map/.*/send-edit-link/")):
            alert.get_by_role("button", name="Send me the link").click()
        assert patched.called
        patched.assert_called_with(
            "The uMap edit link for your map: Untitled map",
            f"Here is your secret edit link: {new_map.get_anonymous_edit_url()}",
            "test@test.org",
            ["foo@bar.com"],
            fail_silently=False,
        )


def test_email_sending_error_are_catched(tilelayer, page, live_server):
    page.goto(f"{live_server.url}/en/map/new")
    alert_creation = page.locator('umap-alert-creation div[role="dialog"]')
    with page.expect_response(re.compile(r".*/map/create/")):
        page.get_by_role("button", name="Save").click()
    alert_creation.get_by_placeholder("Email").fill("foo@bar.com")
    with patch("umap.views.send_mail", side_effect=SMTPException) as patched:
        with page.expect_response(re.compile("/en/map/.*/send-edit-link/")):
            alert_creation.get_by_role("button", name="Send me the link").click()
        assert patched.called
        alert = page.locator('umap-alert div[role="dialog"]')
        expect(alert.get_by_text("Can't send email to foo@bar.com")).to_be_visible()


@pytest.mark.skip(reason="Changing DEFAULT_FROM_EMAIL at runtime has no effect")
def test_alert_message_after_create_show_link_even_without_mail(
    tilelayer, live_server, page, monkeypatch, settings
):
    # Disable email
    settings.DEFAULT_FROM_EMAIL = None
    page.goto(f"{live_server.url}/en/map/new")
    with page.expect_response(re.compile(r".*/map/create/")):
        page.get_by_role("button", name="Save").click()
    alert = page.locator('umap-alert-creation div[role="dialog"]')
    expect(alert).to_be_visible()
    expect(
        alert.get_by_text("Your map has been created with an anonymous account!")
    ).to_be_visible()
    expect(alert.get_by_role("button", name="Copy")).to_be_visible()
    expect(alert.get_by_role("button", name="Send me the link")).to_be_hidden()


def test_anonymous_owner_can_delete_the_map(anonymap, live_server, owner_session):
    assert Map.objects.count() == 1
    owner_session.goto(f"{live_server.url}{anonymap.get_absolute_url()}")
    owner_session.get_by_role("button", name="Edit").click()
    owner_session.get_by_role("button", name="Map advanced properties").click()
    owner_session.get_by_text("Advanced actions").click()
    expect(owner_session.get_by_role("button", name="Delete")).to_be_visible()
    owner_session.get_by_role("button", name="Delete").click()
    with owner_session.expect_response(re.compile(r".*/update/delete/.*")):
        owner_session.get_by_role("button", name="OK").click()
    assert Map.objects.get(pk=anonymap.pk).share_status == Map.DELETED


def test_non_owner_cannot_see_delete_button(anonymap, live_server, page):
    anonymap.edit_status = Map.ANONYMOUS
    anonymap.save()
    page.goto(f"{live_server.url}{anonymap.get_absolute_url()}")
    page.get_by_role("button", name="Edit").click()
    page.get_by_role("button", name="Map advanced properties").click()
    page.get_by_text("Advanced actions").click()
    expect(page.get_by_role("button", name="Delete")).to_be_hidden()



import json
from urllib.parse import quote

import pytest
from playwright.sync_api import expect

pytestmark = pytest.mark.django_db

GEOJSON = {
    "type": "FeatureCollection",
    "features": [
        {
            "type": "Feature",
            "properties": {
                "name": "Niagara Falls",
            },
            "geometry": {
                "type": "Point",
                "coordinates": [-79.04, 43.08],
            },
        }
    ],
}
GEOJSON2 = {
    "type": "FeatureCollection",
    "features": [
        {
            "type": "Feature",
            "properties": {
                "name": "Montmorency Falls",
            },
            "geometry": {
                "type": "Point",
                "coordinates": [-71.14, 46.89],
            },
        }
    ],
}
CSV = "name,latitude,longitude\nNiagara Falls,43.08,-79.04"


def test_map_preview(page, live_server, tilelayer):
    page.goto(f"{live_server.url}/map/")
    # Edit mode is not enabled
    edit_button = page.get_by_role("button", name="Edit")
    expect(edit_button).to_be_visible()


def test_map_preview_can_load_remote_geojson(page, live_server, tilelayer):
    def handle(route):
        route.fulfill(json=GEOJSON)

    # Intercept the route to the proxy
    page.route("*/**/ajax-proxy/**", handle)

    page.goto(f"{live_server.url}/map/?dataUrl=http://some.org/geo.json")
    markers = page.locator(".leaflet-marker-icon")
    expect(markers).to_have_count(1)


def test_map_preview_can_load_mutiple_remote_geojson(page, live_server, tilelayer):
    def handle(route):
        if "2" in route.request.url:
            route.fulfill(json=GEOJSON2)
        else:
            route.fulfill(json=GEOJSON)

    # Intercept the route to the proxy
    page.route("*/**/ajax-proxy/**", handle)

    page.goto(
        (
            f"{live_server.url}/map/?"
            "dataUrl=http://some.org/geo.json&dataUrl=http://some.org/geo2.json"
        )
    )
    markers = page.locator(".leaflet-marker-icon")
    expect(markers).to_have_count(2)


def test_map_preview_can_load_remote_csv(page, live_server, tilelayer):
    def handle(route):
        route.fulfill(body=CSV)

    # Intercept the route to the proxy
    page.route("*/**/ajax-proxy/**", handle)

    page.goto(f"{live_server.url}/map/?dataUrl=http://some.org/geo.csv&dataFormat=csv")
    markers = page.locator(".leaflet-marker-icon")
    expect(markers).to_have_count(1)


def test_map_preview_can_load_geojson_in_querystring(page, live_server, tilelayer):
    page.goto(f"{live_server.url}/map/?data={quote(json.dumps(GEOJSON))}")
    markers = page.locator(".leaflet-marker-icon")
    expect(markers).to_have_count(1)


def test_map_preview_can_load_csv_in_querystring(page, live_server, tilelayer):
    page.goto(f"{live_server.url}/map/?data={quote(CSV)}&dataFormat=csv")
    markers = page.locator(".leaflet-marker-icon")
    expect(markers).to_have_count(1)


def test_map_preview_can_change_styling_from_querystring(page, live_server, tilelayer):
    page.goto(f"{live_server.url}/map/?data={quote(json.dumps(GEOJSON))}&color=DarkRed")
    markers = page.locator(".leaflet-marker-icon .icon_container")
    expect(markers).to_have_count(1)
    expect(markers).to_have_css("background-color", "rgb(139, 0, 0)")


def test_can_open_feature_on_load(page, live_server, tilelayer):
    page.goto(
        f"{live_server.url}/map/?data={quote(json.dumps(GEOJSON))}&feature=Niagara Falls"
    )
    # Popup is open.
    expect(page.get_by_text("Niagara Falls")).to_be_visible()



import pytest
from django.urls import reverse

from umap.models import Map, Team

pytestmark = pytest.mark.django_db


def test_can_see_team_maps(client, map, team):
    map.team = team
    map.save()
    url = reverse("team_maps", args=(team.pk,))
    response = client.get(url)
    assert response.status_code == 200
    assert map.name in response.content.decode()


@pytest.mark.parametrize("share_status", [Map.PRIVATE, Map.DRAFT])
def test_others_cannot_see_team_private_maps_in_team_page(
    client, map, team, user, share_status
):
    map.team = team
    map.share_status = share_status
    map.save()
    url = reverse("team_maps", args=(team.pk,))
    response = client.get(url)
    assert response.status_code == 200
    assert map.name not in response.content.decode()
    # User is not in team
    client.login(username=user.username, password="123123")
    response = client.get(url)
    assert response.status_code == 200
    assert map.name not in response.content.decode()


@pytest.mark.parametrize("share_status", [Map.PRIVATE, Map.DRAFT])
def test_members_can_see_private_maps_in_team_page(
    client, map, team, user, share_status
):
    map.team = team
    map.share_status = share_status
    map.save()
    user.teams.add(team)
    user.save()
    url = reverse("team_maps", args=(team.pk,))
    client.login(username=user.username, password="123123")
    response = client.get(url)
    assert response.status_code == 200
    assert map.name in response.content.decode()


def test_user_can_see_their_teams(client, team, user):
    user.teams.add(team)
    user.save()
    url = reverse("user_teams")
    client.login(username=user.username, password="123123")
    response = client.get(url)
    assert response.status_code == 200
    assert team.name in response.content.decode()


def test_can_create_a_team(client, user):
    assert not Team.objects.count()
    url = reverse("team_new")
    client.login(username=user.username, password="123123")
    response = client.post(url, {"name": "my new team", "members": [user.pk]})
    assert response.status_code == 302
    assert response["Location"] == "/en/me/teams"
    assert Team.objects.count() == 1
    team = Team.objects.first()
    assert team.name == "my new team"
    assert team in user.teams.all()


def test_can_edit_a_team_name(client, user, team):
    user.teams.add(team)
    user.save()
    assert Team.objects.count() == 1
    url = reverse("team_update", args=(team.pk,))
    client.login(username=user.username, password="123123")
    response = client.post(url, {"name": "my new team", "members": [user.pk]})
    assert response.status_code == 302
    assert response["Location"] == "/en/me/teams"
    assert Team.objects.count() == 1
    modified = Team.objects.first()
    assert modified.name == "my new team"
    assert modified in user.teams.all()


def test_can_add_user_to_team(client, user, user2, team):
    user.teams.add(team)
    user.save()
    assert Team.objects.count() == 1
    url = reverse("team_update", args=(team.pk,))
    client.login(username=user.username, password="123123")
    response = client.post(url, {"name": team.name, "members": [user.pk, user2.pk]})
    assert response.status_code == 302
    assert response["Location"] == "/en/me/teams"
    assert Team.objects.count() == 1
    modified = Team.objects.first()
    assert user in modified.users.all()
    assert user2 in modified.users.all()


def test_can_remove_user_from_team(client, user, user2, team):
    user.teams.add(team)
    user.save()
    user2.teams.add(team)
    user2.save()
    assert Team.objects.count() == 1
    url = reverse("team_update", args=(team.pk,))
    client.login(username=user.username, password="123123")
    response = client.post(url, {"name": team.name, "members": [user.pk]})
    assert response.status_code == 302
    assert response["Location"] == "/en/me/teams"
    assert Team.objects.count() == 1
    modified = Team.objects.first()
    assert user in modified.users.all()
    assert user2 not in modified.users.all()


def test_cannot_edit_a_team_if_not_member(client, user, user2, team):
    user.teams.add(team)
    user.save()
    assert Team.objects.count() == 1
    url = reverse("team_update", args=(team.pk,))
    client.login(username=user2.username, password="456456")
    response = client.post(url, {"name": "my new team", "members": [user.pk]})
    assert response.status_code == 403


def test_can_delete_a_team(client, user, team):
    user.teams.add(team)
    user.save()
    assert Team.objects.count() == 1
    url = reverse("team_delete", args=(team.pk,))
    client.login(username=user.username, password="123123")
    response = client.post(url)
    assert response.status_code == 302
    assert response["Location"] == "/en/me/teams"
    assert Team.objects.count() == 0


def test_cannot_delete_a_team_if_not_member(client, user, user2, team):
    user.teams.add(team)
    user.save()
    assert Team.objects.count() == 1
    url = reverse("team_delete", args=(team.pk,))
    client.login(username=user2.username, password="456456")
    response = client.post(url)
    assert response.status_code == 403
    assert Team.objects.count() == 1


def test_cannot_delete_a_team_if_more_than_one_member(client, user, user2, team):
    user.teams.add(team)
    user.save()
    user2.teams.add(team)
    user2.save()
    assert Team.objects.count() == 1
    url = reverse("team_delete", args=(team.pk,))
    client.login(username=user.username, password="123123")
    response = client.post(url)
    assert response.status_code == 400
    assert Team.objects.count() == 1



# Generated by Django 4.2.2 on 2023-10-30 11:27

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("umap", "0014_map_created_at"),
    ]

    operations = [
        migrations.AlterField(
            model_name="pictogram",
            name="pictogram",
            field=models.FileField(upload_to="pictogram"),
        ),
    ]



from typing import List, Literal, Optional, Union

from pydantic import BaseModel, Field, RootModel


class JoinRequest(BaseModel):
    kind: Literal["JoinRequest"] = "JoinRequest"
    token: str
    peer: str
    username: Optional[str] = ""


class OperationMessage(BaseModel):
    """Message sent from one peer to all the others"""

    kind: Literal["OperationMessage"] = "OperationMessage"
    verb: Literal["upsert", "update", "delete", "batch"]
    subject: Literal["map", "datalayer", "feature"]
    metadata: Optional[dict] = None
    key: Optional[str] = None
    operations: Optional[List] = None


class PeerMessage(BaseModel):
    """Message sent from a specific peer to another one"""

    kind: Literal["PeerMessage"] = "PeerMessage"
    sender: str
    recipient: str
    # The message can be whatever the peers want. It's not checked by the server.
    message: dict


class SavedMessage(BaseModel):
    kind: Literal["SavedMessage"] = "SavedMessage"
    lastKnownHLC: str


class Request(RootModel):
    """Any message coming from the websocket should be one of these, and will be rejected otherwise."""

    root: Union[PeerMessage, OperationMessage, SavedMessage] = Field(
        discriminator="kind"
    )


class JoinResponse(BaseModel):
    """Server response containing the list of peers"""

    kind: Literal["JoinResponse"] = "JoinResponse"
    peers: dict
    peer: str


class ListPeersResponse(BaseModel):
    kind: Literal["ListPeersResponse"] = "ListPeersResponse"
    peers: dict



from datetime import datetime, timedelta
from unittest import mock

import pytest
from django.core.management import call_command

from umap.models import DataLayer, Map

from .base import DataLayerFactory, MapFactory

pytestmark = pytest.mark.django_db


def test_empty_trash(user):
    recent = MapFactory(owner=user)
    recent_layer = DataLayerFactory(map=recent)
    deleted_layer = DataLayerFactory(map=recent)
    recent_deleted = MapFactory(owner=user)
    recent_deleted.move_to_trash()
    recent_deleted.save()
    with mock.patch("django.utils.timezone.now") as mocked:
        mocked.return_value = datetime.utcnow() - timedelta(days=8)
        old_deleted = MapFactory(owner=user)
        old_deleted.move_to_trash()
        deleted_layer.move_to_trash()
        old = MapFactory(owner=user)
    assert Map.objects.count() == 4
    assert DataLayer.objects.count() == 2
    call_command("empty_trash", "--days=7", "--dry-run")
    assert Map.objects.count() == 4
    assert DataLayer.objects.count() == 2
    call_command("empty_trash", "--days=9")
    assert Map.objects.count() == 4
    assert DataLayer.objects.count() == 2
    call_command("empty_trash", "--days=7")
    assert not Map.objects.filter(pk=old_deleted.pk)
    assert Map.objects.filter(pk=old.pk)
    assert Map.objects.filter(pk=recent.pk)
    assert Map.objects.filter(pk=recent_deleted.pk)
    assert not DataLayer.objects.filter(pk=deleted_layer.pk)
    assert DataLayer.objects.filter(pk=recent_layer.pk)



import os
import sys
import types

from django.utils.termcolors import colorize

from .base import *  # NOQA, default values

# Allow to override setting from any file, may be out of the PYTHONPATH,
# to make it easier for non python people.
path = os.environ.get("UMAP_SETTINGS")
if not path:
    # Retrocompat
    path = os.path.join("/etc", "umap", "umap.conf")
    if not os.path.exists(path):
        # Retrocompat
        path = os.path.join(os.path.dirname(os.path.realpath(__file__)), "local.py")
        if not os.path.exists(path):
            print(colorize("No valid UMAP_SETTINGS found", fg="yellow"))
            path = None

if path:
    d = types.ModuleType("config")
    d.__file__ = path
    try:
        with open(path) as config_file:
            exec(compile(config_file.read(), path, "exec"), d.__dict__)
    except IOError as e:
        msg = "Unable to import {} from UMAP_SETTINGS".format(path)
        print(colorize(msg, fg="red"))
        sys.exit(e)
    else:
        print("Loaded local config from", path)
        for key in dir(d):
            if key.isupper():
                value = getattr(d, key)
                if key == "UMAP_CUSTOM_TEMPLATES":
                    if "DIRS" in globals()["TEMPLATES"][0]:
                        globals()["TEMPLATES"][0]["DIRS"].insert(0, value)
                    else:
                        globals()["TEMPLATES"][0]["DIRS"] = [value]
                elif key == "UMAP_CUSTOM_STATICS":
                    globals()["STATICFILES_DIRS"].insert(0, value)
                elif key == "UMAP_FEEDBACK_LINK":
                    globals()["UMAP_HELP_URL"] = value
                else:
                    globals()[key] = value



# Generated by Django 2.2 on 2019-04-16 17:57

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("umap", "0006_auto_20190407_0719"),
    ]

    operations = [
        migrations.AlterField(
            model_name="map",
            name="share_status",
            field=models.SmallIntegerField(
                choices=[
                    (1, "everyone (public)"),
                    (2, "anyone with link"),
                    (3, "editors only"),
                    (9, "blocked"),
                ],
                default=1,
                verbose_name="share status",
            ),
        ),
    ]



import re
from copy import deepcopy
from time import sleep

import pytest
from playwright.sync_api import expect

from ..base import DataLayerFactory

pytestmark = pytest.mark.django_db


DATALAYER_DATA = {
    "type": "FeatureCollection",
    "features": [
        {
            "type": "Feature",
            "properties": {
                "name": "one point in france",
                "foo": "point",
                "bar": "one",
                "label": "this is label one",
            },
            "geometry": {"type": "Point", "coordinates": [3.339844, 46.920255]},
        },
        {
            "type": "Feature",
            "properties": {
                "name": "one polygon in greenland",
                "foo": "polygon",
                "bar": "two",
                "label": "this is label two",
            },
            "geometry": {
                "type": "Polygon",
                "coordinates": [
                    [
                        [-41.3, 71.8],
                        [-43.5, 70.8],
                        [-39.3, 70.9],
                        [-37.7, 72.2],
                        [-41.3, 71.8],
                    ]
                ],
            },
        },
        {
            "type": "Feature",
            "properties": {
                "name": "one line in new zeland",
                "foo": "line",
                "bar": "three",
                "label": "this is label three",
            },
            "geometry": {
                "type": "LineString",
                "coordinates": [
                    [176.1, -38.6],
                    [172.9, -43.3],
                    [168.3, -45.2],
                ],
            },
        },
    ],
    "_umap_options": {
        "displayOnLoad": True,
        "browsable": True,
        "name": "Calque 1",
    },
}


@pytest.fixture
def bootstrap(map, live_server):
    map.settings["properties"]["onLoadPanel"] = "databrowser"
    map.save()
    DataLayerFactory(map=map, data=DATALAYER_DATA)


def test_data_browser_should_be_open(live_server, page, bootstrap, map):
    page.goto(f"{live_server.url}{map.get_absolute_url()}")
    panel = page.locator(".panel.left.on")
    expect(panel).to_have_class(re.compile(".*expanded.*"))
    expect(panel.locator(".umap-browser")).to_be_visible()
    expect(page.get_by_text("one point in france")).to_be_visible()
    expect(page.get_by_text("one line in new zeland")).to_be_visible()
    expect(page.get_by_text("one polygon in greenland")).to_be_visible()


def test_data_browser_should_be_filterable(live_server, page, bootstrap, map):
    page.goto(f"{live_server.url}{map.get_absolute_url()}#2/19/-2")
    expect(page.get_by_title("Features in this layer: 3")).to_be_visible()
    markers = page.locator(".leaflet-marker-icon")
    paths = page.locator(".leaflet-overlay-pane path")
    expect(markers).to_have_count(1)
    expect(paths).to_have_count(2)
    page.locator(".filters summary").click()
    filter_ = page.locator("input[name='filter']")
    expect(filter_).to_be_visible()
    filter_.type("poly")
    expect(page.get_by_title("Features in this layer: 1/3")).to_be_visible()
    expect(page.get_by_title("Features in this layer: 1/3")).to_have_text("(1/3)")
    expect(page.get_by_text("one point in france")).to_be_hidden()
    expect(page.get_by_text("one line in new zeland")).to_be_hidden()
    expect(page.get_by_text("one polygon in greenland")).to_be_visible()
    expect(markers).to_have_count(0)  # Hidden by filter
    expect(paths).to_have_count(1)  # Only polygon
    # Empty the filter
    filter_.fill("")
    filter_.blur()
    expect(markers).to_have_count(1)
    expect(paths).to_have_count(2)
    filter_.type("point")
    expect(page.get_by_text("one point in france")).to_be_visible()
    expect(page.get_by_text("one line in new zeland")).to_be_hidden()
    expect(page.get_by_text("one polygon in greenland")).to_be_hidden()
    expect(markers).to_have_count(1)
    expect(paths).to_have_count(0)


def test_filter_uses_layer_setting_if_any(live_server, page, bootstrap, map):
    datalayer = map.datalayer_set.first()
    datalayer.settings["labelKey"] = "foo"
    datalayer.save()
    page.goto(f"{live_server.url}{map.get_absolute_url()}#2/19/-2")
    expect(page.get_by_title("Features in this layer: 3")).to_be_visible()
    markers = page.locator(".leaflet-marker-icon")
    paths = page.locator(".leaflet-overlay-pane path")
    expect(markers).to_have_count(1)
    expect(paths).to_have_count(2)
    expect(page.get_by_text("point")).to_be_visible()
    expect(page.get_by_text("polygon")).to_be_visible()
    expect(page.get_by_text("line")).to_be_visible()
    page.locator(".filters summary").click()
    filter_ = page.locator("input[name='filter']")
    expect(filter_).to_be_visible()
    filter_.type("po")
    expect(page.get_by_title("Features in this layer: 2/3")).to_be_visible()
    expect(page.get_by_title("Features in this layer: 2/3")).to_have_text("(2/3)")
    expect(page.get_by_text("line")).to_be_hidden()
    expect(page.get_by_text("point")).to_be_visible()
    expect(page.get_by_text("polygon")).to_be_visible()
    expect(markers).to_have_count(1)
    expect(paths).to_have_count(1)  # Only polygon
    # Empty the filter
    filter_.fill("")
    filter_.blur()
    expect(markers).to_have_count(1)
    expect(paths).to_have_count(2)
    filter_.type("point")
    expect(page.get_by_text("point")).to_be_visible()
    expect(page.get_by_text("line")).to_be_hidden()
    expect(page.get_by_text("polygon")).to_be_hidden()
    expect(markers).to_have_count(1)
    expect(paths).to_have_count(0)


def test_filter_works_with_variable_in_labelKey(live_server, page, map):
    map.settings["properties"]["onLoadPanel"] = "databrowser"
    map.save()
    data = deepcopy(DATALAYER_DATA)
    data["_umap_options"]["labelKey"] = "{name} ({bar})"
    DataLayerFactory(map=map, data=data)
    page.goto(f"{live_server.url}{map.get_absolute_url()}#2/19/-2")
    expect(page.get_by_title("Features in this layer: 3")).to_be_visible()
    markers = page.locator(".leaflet-marker-icon")
    paths = page.locator(".leaflet-overlay-pane path")
    expect(markers).to_have_count(1)
    expect(paths).to_have_count(2)
    expect(page.get_by_text("one point in france (one)")).to_be_visible()
    expect(page.get_by_text("one line in new zeland (three)")).to_be_visible()
    expect(page.get_by_text("one polygon in greenland (two)")).to_be_visible()
    page.locator(".filters summary").click()
    filter_ = page.locator("input[name='filter']")
    expect(filter_).to_be_visible()
    filter_.type("two")
    expect(page.get_by_title("Features in this layer: 1/3")).to_be_visible()
    expect(page.get_by_title("Features in this layer: 1/3")).to_have_text("(1/3)")
    expect(page.get_by_text("one polygon in greenland (two)")).to_be_visible()
    expect(page.get_by_text("one line in new zeland (three)")).to_be_hidden()
    expect(page.get_by_text("one point in france (one)")).to_be_hidden()
    expect(markers).to_have_count(0)
    expect(paths).to_have_count(1)  # Only polygon


def test_filter_works_with_missing_name(live_server, page, map):
    map.settings["properties"]["onLoadPanel"] = "databrowser"
    map.save()
    data = deepcopy(DATALAYER_DATA)
    del data["features"][0]["properties"]["name"]
    DataLayerFactory(map=map, data=data, name="foobar")
    page.goto(f"{live_server.url}{map.get_absolute_url()}#2/19/-2")
    expect(page.get_by_title("Features in this layer: 3")).to_be_visible()
    markers = page.locator(".leaflet-marker-icon")
    paths = page.locator(".leaflet-overlay-pane path")
    expect(markers).to_have_count(1)
    expect(paths).to_have_count(2)
    page.locator(".filters summary").click()
    filter_ = page.locator("input[name='filter']")
    expect(filter_).to_be_visible()
    filter_.type("foob")
    expect(markers).to_have_count(1)
    expect(paths).to_have_count(0)


def test_data_browser_can_show_only_visible_features(live_server, page, bootstrap, map):
    # Zoom on France
    page.goto(f"{live_server.url}{map.get_absolute_url()}#6/51.000/2.000")
    page.locator(".filters summary").click()
    el = page.get_by_text("Current map view")
    expect(el).to_be_visible()
    el.click()
    expect(page.get_by_text("one point in france")).to_be_visible()
    expect(page.get_by_text("one line in new zeland")).to_be_hidden()
    expect(page.get_by_text("one polygon in greenland")).to_be_hidden()


def test_data_browser_can_mix_filter_and_bbox(live_server, page, bootstrap, map):
    # Zoom on north west
    page.goto(f"{live_server.url}{map.get_absolute_url()}#4/61.98/-2.68")
    page.locator(".filters summary").click()
    el = page.get_by_text("Current map view")
    expect(el).to_be_visible()
    el.click()
    expect(page.get_by_text("one point in france")).to_be_visible()
    expect(page.get_by_text("one polygon in greenland")).to_be_visible()
    expect(page.get_by_text("one line in new zeland")).to_be_hidden()
    el = page.locator("input[name='filter']")
    expect(el).to_be_visible()
    el.type("poly")
    expect(page.get_by_text("one polygon in greenland")).to_be_visible()
    expect(page.get_by_text("one point in france")).to_be_hidden()
    expect(page.get_by_text("one line in new zeland")).to_be_hidden()


def test_data_browser_bbox_limit_should_be_dynamic(live_server, page, bootstrap, map):
    # Zoom on Europe
    page.goto(f"{live_server.url}{map.get_absolute_url()}#6/51.000/2.000")
    page.locator(".filters summary").click()
    el = page.get_by_text("Current map view")
    expect(el).to_be_visible()
    el.click()
    expect(page.get_by_text("one point in france")).to_be_visible()
    expect(page.get_by_text("one line in new zeland")).to_be_hidden()
    expect(page.get_by_text("one polygon in greenland")).to_be_hidden()
    unzoom = page.get_by_role("button", name="Zoom out")
    expect(unzoom).to_be_visible()
    # Unzoom until we see the Greenland
    unzoom.click()
    sleep(0.5)  # Zooming is async
    unzoom.click()
    sleep(0.5)  # Zooming is async
    unzoom.click()
    sleep(0.5)  # Zooming is async
    expect(page.get_by_text("one point in france")).to_be_visible()
    expect(page.get_by_text("one polygon in greenland")).to_be_visible()
    expect(page.get_by_text("one line in new zeland")).to_be_hidden()


def test_data_browser_bbox_filter_should_be_persistent(
    live_server, page, bootstrap, map
):
    # Zoom on Europe
    page.goto(f"{live_server.url}{map.get_absolute_url()}#6/51.000/2.000")
    page.locator(".filters summary").click()
    el = page.get_by_text("Current map view")
    expect(el).to_be_visible()
    el.click()
    browser = page.locator(".panel.left.on")
    expect(browser.get_by_text("one point in france")).to_be_visible()
    expect(browser.get_by_text("one line in new zeland")).to_be_hidden()
    expect(browser.get_by_text("one polygon in greenland")).to_be_hidden()
    # Close and reopen the browser to make sure this settings is persistent
    close = browser.get_by_title("Close")
    close.click()
    expect(browser).to_be_hidden()
    sleep(0.5)
    expect(browser.get_by_text("one point in france")).to_be_hidden()
    expect(browser.get_by_text("one line in new zeland")).to_be_hidden()
    expect(browser.get_by_text("one polygon in greenland")).to_be_hidden()
    page.get_by_title("Open browser").click()
    expect(browser.get_by_text("one point in france")).to_be_visible()
    expect(browser.get_by_text("one line in new zeland")).to_be_hidden()
    expect(browser.get_by_text("one polygon in greenland")).to_be_hidden()


def test_data_browser_bbox_filtered_is_clickable(live_server, page, bootstrap, map):
    popup = page.locator(".leaflet-popup")
    # Zoom on Europe
    page.goto(f"{live_server.url}{map.get_absolute_url()}#6/51.000/2.000")
    page.locator(".filters summary").click()
    el = page.get_by_text("Current map view")
    expect(el).to_be_visible()
    el.click()
    browser = page.locator(".panel.left.on")
    expect(browser.get_by_text("one point in france")).to_be_visible()
    expect(browser.get_by_text("one line in new zeland")).to_be_hidden()
    expect(browser.get_by_text("one polygon in greenland")).to_be_hidden()
    expect(popup).to_be_hidden()
    browser.get_by_text("one point in france").click()
    expect(popup).to_be_visible()
    expect(popup.get_by_text("one point in france")).to_be_visible()


def test_data_browser_with_variable_in_name(live_server, page, bootstrap, map):
    # Include a variable
    map.settings["properties"]["labelKey"] = "{name} ({foo})"
    map.save()
    page.goto(f"{live_server.url}{map.get_absolute_url()}#2/19/-2")
    expect(page.get_by_text("one point in france (point)")).to_be_visible()
    expect(page.get_by_text("one line in new zeland (line)")).to_be_visible()
    expect(page.get_by_text("one polygon in greenland (polygon)")).to_be_visible()
    page.locator(".filters summary").click()
    filter_ = page.locator("input[name='filter']")
    expect(filter_).to_be_visible()
    filter_.type("foobar")  # Hide all
    expect(page.get_by_text("one point in france (point)")).to_be_hidden()
    expect(page.get_by_text("one line in new zeland (line)")).to_be_hidden()
    expect(page.get_by_text("one polygon in greenland (polygon)")).to_be_hidden()
    # Empty back the filter
    filter_.fill("")
    filter_.blur()
    expect(page.get_by_text("one point in france (point)")).to_be_visible()
    expect(page.get_by_text("one line in new zeland (line)")).to_be_visible()
    expect(page.get_by_text("one polygon in greenland (polygon)")).to_be_visible()


def test_should_sort_features_in_natural_order(live_server, map, page):
    map.settings["properties"]["onLoadPanel"] = "databrowser"
    map.save()
    datalayer_data = deepcopy(DATALAYER_DATA)
    datalayer_data["features"][0]["properties"]["name"] = "9. a marker"
    datalayer_data["features"][1]["properties"]["name"] = "1. a poly"
    datalayer_data["features"][2]["properties"]["name"] = "100. a line"
    DataLayerFactory(map=map, data=datalayer_data)
    page.goto(f"{live_server.url}{map.get_absolute_url()}")
    features = page.locator(".umap-browser .datalayer li")
    expect(features).to_have_count(3)
    expect(features.nth(0)).to_have_text("1. a poly")
    expect(features.nth(1)).to_have_text("9. a marker")
    expect(features.nth(2)).to_have_text("100. a line")


def test_should_redraw_list_on_feature_delete(live_server, openmap, page, bootstrap):
    page.goto(f"{live_server.url}{openmap.get_absolute_url()}#2/19/-2")
    # Enable edit
    page.get_by_role("button", name="Edit").click()
    buttons = page.locator(".umap-browser .datalayer li .icon-delete")
    expect(buttons).to_have_count(3)
    buttons.first.click()
    expect(buttons).to_have_count(2)
    page.locator(".edit-undo").click()
    expect(buttons).to_have_count(3)


def test_should_show_header_for_display_on_load_false(
    live_server, page, bootstrap, map, datalayer
):
    datalayer.settings["displayOnLoad"] = False
    datalayer.settings["name"] = "This layer is not loaded"
    datalayer.save()
    page.goto(f"{live_server.url}{map.get_absolute_url()}")
    browser = page.locator(".umap-browser")
    expect(browser).to_be_visible()
    expect(browser.get_by_text("This layer is not loaded")).to_be_visible()


def test_should_use_color_variable(live_server, map, page):
    map.settings["properties"]["onLoadPanel"] = "databrowser"
    map.settings["properties"]["color"] = "{mycolor}"
    map.save()
    datalayer_data = deepcopy(DATALAYER_DATA)
    datalayer_data["features"][0]["properties"]["mycolor"] = "DarkRed"
    datalayer_data["features"][2]["properties"]["mycolor"] = "DarkGreen"
    DataLayerFactory(map=map, data=datalayer_data)
    page.goto(f"{live_server.url}{map.get_absolute_url()}")
    features = page.locator(".umap-browser .datalayer li .feature-color")
    expect(features).to_have_count(3)
    # DarkGreen
    expect(features.nth(0)).to_have_css("background-color", "rgb(0, 100, 0)")
    # DarkRed
    expect(features.nth(1)).to_have_css("background-color", "rgb(139, 0, 0)")
    # DarkBlue (default color)
    expect(features.nth(2)).to_have_css("background-color", "rgb(0, 0, 139)")


def test_should_allow_to_toggle_datalayer_visibility(live_server, map, page, bootstrap):
    page.goto(f"{live_server.url}{map.get_absolute_url()}#2/19/-2")
    markers = page.locator(".leaflet-marker-icon")
    paths = page.locator(".leaflet-overlay-pane path")
    expect(markers).to_have_count(1)
    expect(paths).to_have_count(2)
    toggle = page.locator(".umap-browser").get_by_title("Show/hide layer")
    toggle.click()
    expect(markers).to_have_count(0)
    expect(paths).to_have_count(0)


def test_should_have_edit_buttons_in_edit_mode(live_server, openmap, page, bootstrap):
    page.goto(f"{live_server.url}{openmap.get_absolute_url()}")
    browser = page.locator(".umap-browser")
    edit_layer = browser.get_by_title("Edit", exact=True)
    in_table = browser.get_by_title("Edit properties in a table")
    delete_layer = browser.get_by_title("Delete layer")
    edit_feature = browser.get_by_title("Edit this feature")
    delete_feature = browser.get_by_title("Delete this feature")
    expect(edit_layer).to_be_hidden()
    expect(in_table).to_be_hidden()
    expect(delete_layer).to_be_hidden()
    # Does not work
    # to_have_count does not seem to case about the elements being visible or not
    # and to_be_hidden is not happy if the selector resolve to more than on element
    # expect(edit_feature).to_have_count(0)
    # expect(delete_feature).to_be_hidden()
    # Switch to edit mode
    page.get_by_role("button", name="Edit").click()
    expect(edit_layer).to_be_visible()
    expect(in_table).to_be_visible()
    expect(delete_layer).to_be_visible()
    expect(edit_feature).to_have_count(3)
    expect(delete_feature).to_have_count(3)


def test_main_toolbox_toggle_all_layers(live_server, map, page):
    map.settings["properties"]["onLoadPanel"] = "databrowser"
    map.save()
    data = {
        "type": "FeatureCollection",
        "features": [
            {
                "type": "Feature",
                "properties": {"name": "one point"},
                "geometry": {"type": "Point", "coordinates": [3.33, 46.92]},
            },
        ],
    }
    DataLayerFactory(map=map, data=data)
    data = {
        "type": "FeatureCollection",
        "features": [
            {
                "type": "Feature",
                "properties": {"name": "one other point"},
                "geometry": {"type": "Point", "coordinates": [3.34, 46.94]},
            },
        ],
    }
    DataLayerFactory(map=map, data=data)
    data = {
        "type": "FeatureCollection",
        "features": [
            {
                "type": "Feature",
                "properties": {"name": "another point"},
                "geometry": {"type": "Point", "coordinates": [3.35, 46.95]},
            },
        ],
        "_umap_options": {"displayOnLoad": False},
    }
    DataLayerFactory(map=map, data=data, settings={"displayOnLoad": False})
    page.goto(f"{live_server.url}{map.get_absolute_url()}#10/46.93/3.33")
    markers = page.locator(".leaflet-marker-icon")
    expect(markers).to_have_count(2)
    # Only one is off
    expect(page.locator(".datalayer.off")).to_have_count(1)

    # Click on button
    page.locator(".umap-browser [data-ref=toggle]").click()
    # Should have hidden the two other layers
    expect(page.locator(".datalayer.off")).to_have_count(3)
    expect(markers).to_have_count(0)

    # Click again
    page.locator(".umap-browser [data-ref=toggle]").click()
    # Should shown all layers
    expect(page.locator(".datalayer.off")).to_have_count(0)
    expect(markers).to_have_count(3)

    # Click again
    page.locator(".umap-browser [data-ref=toggle]").click()
    # Should hidden again all layers
    expect(page.locator(".datalayer.off")).to_have_count(3)
    expect(markers).to_have_count(0)


def test_honour_the_label_fields_settings(live_server, map, page, bootstrap, settings):
    settings.UMAP_LABEL_KEYS = ["label", "name"]
    page.goto(f"{live_server.url}{map.get_absolute_url()}")
    expect(page.locator(".panel").get_by_text("this is label one")).to_be_visible()
    expect(page.locator(".panel").get_by_text("this is label two")).to_be_visible()
    expect(page.locator(".panel").get_by_text("this is label three")).to_be_visible()



import re

import pytest
from playwright.sync_api import expect

from ..base import DataLayerFactory

pytestmark = pytest.mark.django_db


def test_caption(live_server, page, map):
    map.settings["properties"]["onLoadPanel"] = "caption"
    map.save()
    basic = DataLayerFactory(map=map, name="Basic layer")
    non_loaded = DataLayerFactory(
        map=map, name="Non loaded", settings={"displayOnLoad": False}
    )
    hidden = DataLayerFactory(map=map, name="Hidden", settings={"inCaption": False})
    page.goto(f"{live_server.url}{map.get_absolute_url()}")
    panel = page.locator(".panel.left.on")
    expect(panel).to_have_class(re.compile(".*condensed.*"))
    expect(panel.locator(".umap-caption")).to_be_visible()
    expect(panel.locator(".caption-item").get_by_text(basic.name)).to_be_visible()
    expect(
        panel.locator(".caption-item .off").get_by_text(non_loaded.name)
    ).to_be_visible()
    expect(panel.locator(".caption-item").get_by_text(hidden.name)).to_be_hidden()
    expect(panel.get_by_text("Created at")).to_be_visible()


def test_caption_should_display_owner_as_author(live_server, page, map):
    map.settings["properties"]["onLoadPanel"] = "caption"
    map.save()
    page.goto(f"{live_server.url}{map.get_absolute_url()}")
    panel = page.locator(".panel.left.on")
    expect(panel).to_be_visible()
    expect(panel.get_by_text("By Gabriel")).to_be_visible()


def test_caption_should_display_team_as_author(live_server, page, map, team):
    map.settings["properties"]["onLoadPanel"] = "caption"
    map.team = team
    map.save()
    page.goto(f"{live_server.url}{map.get_absolute_url()}")
    panel = page.locator(".panel.left.on")
    expect(panel).to_be_visible()
    expect(panel.get_by_text("By Gabriel")).to_be_hidden()
    expect(panel.get_by_text("By Awesome Team")).to_be_visible()



"""Base settings shared by all environments"""

# Import global settings to make it easier to extend settings.
import os
from email.utils import parseaddr

import environ
from django.conf.locale import LANG_INFO
from django.utils.translation import gettext_lazy as _

import umap as project_module

env = environ.Env()

# =============================================================================
# Generic Django project settings
# =============================================================================


INTERNAL_IPS = env.list("INTERNAL_IPS", default=["127.0.0.1"])
ALLOWED_HOSTS = env.list("ALLOWED_HOSTS", default=["*"])
ADMINS = tuple(parseaddr(email) for email in env.list("ADMINS", default=[]))
ASGI_APPLICATION = "umap.asgi.application"

DEBUG = env.bool("DEBUG", default=False)

SITE_ID = 1
# Add languages we're missing from Django
LANG_INFO.update(
    {
        "am-et": {
            "bidi": False,
            "name": "Amharic",
            "code": "am-et",
            "name_local": "አማርኛ",
        },
        "zh": {
            "bidi": False,
            "code": "zh",
            "name": "Chinese",
            "name_local": "简体中文",
        },
        "si": {
            "bidi": False,
            "code": "si",
            "name": "Sinhala",
            "name_local": "සිංහල",
        },
        "ms": {
            "bidi": False,
            "code": "ms",
            "name": "Malay",
            "name_local": "Bahasa Melayu",
        },
        "fa-ir": {
            "bidi": True,
            "code": "fa-ir",
            "name": "Persian (Iran)",
            "name_local": "فارسی",
        },
    }
)
# Local time zone for this installation. Choices can be found here:
# http://en.wikipedia.org/wiki/List_of_tz_zones_by_name
TIME_ZONE = "UTC"
USE_TZ = True
USE_I18N = True
LANGUAGE_CODE = "en"
LANGUAGES = (
    ("am-et", "Amharic"),
    ("ar", "Arabic"),
    ("ast", "Asturian"),
    ("bg", "Bulgarian"),
    ("br", "Breton"),
    ("ca", "Catalan"),
    ("cs-cz", "Czech"),
    ("da", "Danish"),
    ("de", "German"),
    ("el", "Greek"),
    ("en", "English"),
    ("es", "Spanish"),
    ("et", "Estonian"),
    ("eu", "Basque"),
    ("fa-ir", "Persian (Iran)"),
    ("fi", "Finnish"),
    ("fr", "French"),
    ("gl", "Galician"),
    ("he", "Hebrew"),
    ("hr", "Croatian"),
    ("hu", "Hungarian"),
    ("id", "Indonesian"),
    ("is", "Icelandic"),
    ("it", "Italian"),
    ("ja", "Japanese"),
    ("ko", "Korean"),
    ("lt", "Lithuanian"),
    ("ms", "Malay"),
    ("nl", "Dutch"),
    ("no", "Norwegian"),
    ("pl", "Polish"),
    ("pt", "Portuguese"),
    ("pt-br", "Portuguese (Brazil)"),
    ("pt-pt", "Portuguese (Portugal)"),
    ("ro", "Romanian"),
    ("ru", "Russian"),
    ("si", "Sinhala"),
    ("sk-sk", "Slovak"),
    ("sl", "Slovenian"),
    ("sr", "Serbian"),
    ("sv", "Swedish"),
    ("th-th", "Thai (Thailand)"),
    ("tr", "Turkish"),
    ("uk-ua", "Ukrainian"),
    ("vi", "Vietnamese"),
    ("zh", "Chinese"),
    ("zh-tw", "Chinese (Taiwan)"),
)

# Make this unique, and don't share it with anybody.
SECRET_KEY = env("SECRET_KEY", default=None)

INSTALLED_APPS = (
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.sites",
    "django.contrib.messages",
    "django.contrib.staticfiles",
    "django.contrib.admin",
    "django.contrib.gis",
    "django_probes",
    "umap",
    "social_django",
    # See https://github.com/peopledoc/django-agnocomplete/commit/26eda2dfa4a2f8a805ca2ea19a0c504b9d773a1c
    # Django does not find the app config in the default place, so the app is not loaded
    # so the "autodiscover" is not run.
    "agnocomplete.app.AgnocompleteConfig",
)
DEFAULT_AUTO_FIELD = "django.db.models.AutoField"

EMAIL_BACKEND = "django.core.mail.backends.console.EmailBackend"
DEFAULT_FROM_EMAIL = None
# https://docs.djangoproject.com/en/4.2/releases/4.1/#forms
FORM_RENDERER = "django.forms.renderers.DjangoTemplates"

# =============================================================================
# Calculation of directories relative to the project module location
# =============================================================================


PROJECT_DIR = os.path.dirname(os.path.realpath(project_module.__file__))

# =============================================================================
# Project URLS and media settings
# =============================================================================

ROOT_URLCONF = "umap.urls"
WSGI_APPLICATION = "umap.wsgi.application"

LOGIN_URL = "/login/"
LOGOUT_URL = "/logout/"
LOGIN_REDIRECT_URL = "login_popup_end"

STATIC_URL = "/static/"
MEDIA_URL = "/uploads/"

STATIC_ROOT = os.path.join("static")
MEDIA_ROOT = os.path.join("uploads")

STATICFILES_FINDERS = [
    "django.contrib.staticfiles.finders.FileSystemFinder",
    "django.contrib.staticfiles.finders.AppDirectoriesFinder",
]
STATICFILES_DIRS = []  # May be extended when using UMAP_CUSTOM_STATICS
STORAGES = {
    "default": {
        "BACKEND": "django.core.files.storage.FileSystemStorage",
    },
    "data": {
        "BACKEND": "umap.storage.fs.FSDataStorage",
    },
    "staticfiles": {
        "BACKEND": "umap.storage.staticfiles.UmapManifestStaticFilesStorage",
    },
}
# Add application/json and application/geo+json to default django-storages setting
# in order to gzip our datalayers geojson files.
GZIP_CONTENT_TYPES = [
    "text/css",
    "text/javascript",
    "application/javascript",
    "application/x-javascript",
    "image/svg+xml",
    "application/json",
    "application/geo+json",
]


# =============================================================================
# Templates
# =============================================================================

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": (
                "django.contrib.auth.context_processors.auth",
                "django.template.context_processors.debug",
                "django.template.context_processors.i18n",
                "django.template.context_processors.request",
                "django.template.context_processors.media",
                "django.template.context_processors.static",
                "django.template.context_processors.tz",
                "django.contrib.messages.context_processors.messages",
                "social_django.context_processors.backends",
                "social_django.context_processors.login_redirect",
                "umap.context_processors.settings",
                "umap.context_processors.version",
            )
        },
    },
]


# =============================================================================
# Middleware
# =============================================================================

MIDDLEWARE = (
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.locale.LocaleMiddleware",
    "django.middleware.common.CommonMiddleware",
    "umap.middleware.readonly_middleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "umap.middleware.deprecated_auth_backend",
)


# =============================================================================
# Auth / security
# =============================================================================

# Set to True if login into django account should be possible. Default is to
# only use OAuth flow.
ENABLE_ACCOUNT_LOGIN = env.bool("ENABLE_ACCOUNT_LOGIN", default=False)
USER_DISPLAY_NAME = "{username}"
# For use by Agnocomplete
# See https://django-agnocomplete.readthedocs.io/en/latest/autocomplete-definition.html#agnocompletemode
USER_AUTOCOMPLETE_FIELDS = ["^username"]
USER_URL_FIELD = "username"

# =============================================================================
# Miscellaneous project settings
# =============================================================================
UMAP_ALLOW_ANONYMOUS = env.bool("UMAP_ALLOW_ANONYMOUS", default=False)
UMAP_ALLOW_EDIT_PROFILE = env.bool("UMAP_ALLOW_EDIT_PROFILE", default=True)

UMAP_EXTRA_URLS = {
    "routing": "http://www.openstreetmap.org/directions?engine=osrm_car&route={lat},{lng}&locale={locale}#map={zoom}/{lat}/{lng}",  # noqa
    "ajax_proxy": "/ajax-proxy/?url={url}&ttl={ttl}",
    "search": "https://photon.komoot.io/api/?",
    "edit_in_osm": "https://www.openstreetmap.org/edit#map={zoom}/{lat}/{lng}",
}
UMAP_KEEP_VERSIONS = env.int("UMAP_KEEP_VERSIONS", default=10)
SITE_URL = env("SITE_URL", default="http://umap.org")
SHORT_SITE_URL = env("SHORT_SITE_URL", default=None)
SITE_NAME = "uMap"
SITE_DESCRIPTION = "Online map creator"
UMAP_DEMO_SITE = env("UMAP_DEMO_SITE", default=False)
UMAP_EXCLUDE_DEFAULT_MAPS = False
UMAP_MAPS_PER_PAGE = 5
UMAP_MAPS_PER_SEARCH = 25
UMAP_MAPS_PER_PAGE_OWNER = 10
UMAP_SEARCH_CONFIGURATION = "simple"
UMAP_HELP_URL = "https://discover.umap-project.org/"
USER_MAPS_URL = "user_maps"
DATABASES = {
    "default": env.db(
        default="postgis://localhost:5432/umap",
        engine="django.contrib.gis.db.backends.postgis",
    )
}
UMAP_DEFAULT_SHARE_STATUS = None
UMAP_DEFAULT_EDIT_STATUS = None
UMAP_DEFAULT_FEATURES_HAVE_OWNERS = False
UMAP_HOME_FEED = "latest"
UMAP_IMPORTERS = {}
UMAP_HOST_INFOS = {}
UMAP_LABEL_KEYS = ["name", "title"]
UMAP_TAGS = (
    ("arts", _("Art and Culture")),
    ("cycling", _("Cycling")),
    ("business", _("Business")),
    ("environment", _("Environment")),
    ("education", _("Education")),
    ("food", _("Food and Agriculture")),
    ("geopolitics", _("Geopolitics")),
    ("health", _("Health")),
    ("hiking", _("Hiking")),
    ("history", _("History")),
    ("public", _("Public sector")),
    ("science", _("Science")),
    ("shopping", _("Shopping")),
    ("sport", _("Sport and Leisure")),
    ("travel", _("Travel")),
    ("transports", _("Transports")),
    ("tourism", _("Tourism")),
)

UMAP_READONLY = env("UMAP_READONLY", default=False)
UMAP_GZIP = True
LOCALE_PATHS = [os.path.join(PROJECT_DIR, "locale")]

LEAFLET_LONGITUDE = env.int("LEAFLET_LONGITUDE", default=2)
LEAFLET_LATITUDE = env.int("LEAFLET_LATITUDE", default=51)
LEAFLET_ZOOM = env.int("LEAFLET_ZOOM", default=6)


# =============================================================================
# Third party app settings
# =============================================================================
LOGIN_URL = "login"
SOCIAL_AUTH_LOGIN_REDIRECT_URL = "/login/popup/end/"

AUTHENTICATION_BACKENDS = ()
DEPRECATED_AUTHENTICATION_BACKENDS = []

SOCIAL_AUTH_OPENSTREETMAP_OAUTH2_KEY = env(
    "SOCIAL_AUTH_OPENSTREETMAP_OAUTH2_KEY", default=""
)
SOCIAL_AUTH_OPENSTREETMAP_OAUTH2_SECRET = env(
    "SOCIAL_AUTH_OPENSTREETMAP_OAUTH2_SECRET", default=""
)
if SOCIAL_AUTH_OPENSTREETMAP_OAUTH2_KEY and SOCIAL_AUTH_OPENSTREETMAP_OAUTH2_SECRET:
    AUTHENTICATION_BACKENDS += (
        "social_core.backends.openstreetmap_oauth2.OpenStreetMapOAuth2",
    )

AUTHENTICATION_BACKENDS += ("django.contrib.auth.backends.ModelBackend",)

LOGGING = {
    "version": 1,
    "disable_existing_loggers": False,
    "handlers": {
        "console": {
            "level": "ERROR",
            "filters": None,
            "class": "logging.StreamHandler",
        },
    },
    "loggers": {
        "django": {
            "handlers": ["console"],
            "level": "ERROR",
        },
    },
}

# Real-time editing configuration

REALTIME_ENABLED = env.bool("REALTIME_ENABLED", default=False)

REDIS_URL = env("REDIS_URL", default="redis://localhost:6379")
